#' TendonMCMCSampling: Perform RWM MCMC on tendon stress-strain data
#'
#' The TendonMCMCSampling package provides functions in both R and RCpp to 
#' perform Random-Walk Metropolis Markov Chain Monte Carlo on experimental
#' stress-strain data taken from tendons. Important folders in the package
#' are detailed below. This package has not been uploaded publicly to GitHub
#' or CRAN yet. So far, it has been run on RStudio on a Windows computer. To
#' run the package in RStudio, open the .Rproj file in the package's folder and
#' then on the Build tab in the upper right corner of the RStudio screen, click
#' the Install and Restart button to run the package.
#' 
#' Documentation of the package files and functions and naming of the 
#' functions have been made to try and adhere to the style guide:
#' https://r-pkgs.org/
#' 
#' @section R:
#' This folder contains the functions created in R that are needed to perform
#' MCMC on tendon stress-strain data and create plots from the samples we 
#' obtain. Some of the R files contain a group of functions that work towards
#' an overarching goal. The groups of functions are as follows:
#' 
#' \itemize{
#'   \item The various functions used to calculate the engineering stress are
#'     located in the file stress_functions.R.
#'   \item The two functions involved in transforming one set of parameters to
#'     another are located in parameter_transformations.R.
#'   \item The three functions involved in calculating the transition 
#'     probability are located in probability_functions.R.
#'   \item The functions used to create both the ggplot matrices of marginal 
#'     posteriors, 2D contour plots, and parameter correlations, and the plots
#'     of the mean and confidence bands of the predicted stresses of a subset
#'     of parameter vectors are located in plotting_functions.R
#' }
#' Other functions, which possess a more distinct purpose compared to the other 
#' functions are located in their own files. Other files within the folder R
#' contain documentation of the data sets and the package. To view the 
#' documentation of a function in the R console, type ?<FUNCTION_NAME> 
#' E.g. ?compute_cred_int. (Do not include the full stop.)
#' 
#' @section src:
#' This folder contains versions of most of the functions detailed in R, but 
#' instead written in RCpp/C++, in order to have fast versions of the functions. 
#' Functions that are found in R, but do not have an equivalent version written
#' in C++ include
#' 
#' \itemize{
#'   \item The function that determines the means and standard deviations of 
#'     the normal prior distributions (for the transformed parameters) that are
#'     used in the algorithm to calculate the prior-probability ratio.
#'   \item The plotting functions, as all plotting is performed in R using 
#'     ggplot (and other plotting packages in R).
#'   \item The function that computes the credible interval using a data frame
#'     of parameter values.
#' }
#' For each function, there is an associated header file. Each header file is 
#' stored in its own separate file, even though some functions are grouped into
#' their own document. To view function documentation, just do the same as for
#' the functions in R. E.g. ?rcpp_calc_ave_rel_err.  
#' 
#' @section data:
#' This folder contains the experimental data sets that are fit to in the RWM
#' MCMC algorithm. To learn more about the data sets, type ?<DATA SET NAME> into
#' the R console. E.g. ?h16_data. (Do not include the full stop.)
#' 
#' @section demo:
#' This folder in the package contains scripts that can be adapted and then run
#' to perform MCMC on experimental or synthetic tendon stress-strain data. Of 
#' the ten scripts, five perform RWM MCMC with the functions written in R, and
#' five perform RWM MCMC with the functions written in RCpp/C++. The five
#' scripts for sampling in R or RCpp/C++ correspond to one script for each of 
#' the four experimental data sets, and one for the synthetic data set. Figures,
#' data frames, and vectors produced in the scripts that use R functions are
#' denoted by '_rscript' at the end of their name. Figures, data frames, and 
#' vectors produced by the scripts that use RCpp/C++ functions do not possess
#' a suffix at the end of their name.
#'
#' @docType package
#' @name TendonMCMCSampling
NULL