#' Add an accepted parameter vector to the matrix of the closest fits
#' 
#' Each time a proposed parameter vector is accepted in the Random-Walk 
#' Metropolis Markov chain Monte Carlo algorithm, we ascertain the average
#' relative error to the experimental data set. If the error is less than one
#' of the vectors in the matrix, then the newly accepted parameter vector
#' replaces the vector in the matrix that produces the worst fit to data.
#'
#' @param current_matrix A matrix of the accepted parameter vectors that
#'   achieve the best fits to experimental data in the algorithm so far.
#' @param n_row The number of parameter vectors that we save in the matrix.
#' @param ave_error The average relative error between the experimental data
#'   and the predicted stresses found with the accepted parameter vector.
#' @param iter An integer, the iteration in the algorithm that the parameter
#'   vector was accepted.
#' @param param_vec The accepted parameter vector.
#'
#' @return A matrix of the transformed parameter vectors that have been
#'   accepted and produced the best n_row fits to data so far in the algorithm.
#' @export
alter_closest_fits_mat <- function(current_matrix,
                                   n_row,
                                   ave_error,
                                   iter,
                                   param_vec) {
  if (ave_error < max(current_matrix[, 2])) {
    current_matrix[n_row, ] <- c(
      iter,
      ave_error,
      param_vec
    )
    current_matrix <-
      current_matrix[order(current_matrix[, 2], decreasing = FALSE), ]
  }
  return(current_matrix)
}
