#' Compute the credible interval for parameter values stored in a data frame
#' 
#' In this function we calculate the credible intervals for the parameters of
#' the model. The percentage credibility is specified in the function call.
#' If the upper and lower indices are not integers, then we calculate a
#  weighted estimate of the lower and upper bounds of the credible interval.
#  E.g. if upper_index is equal to 957.75, then we estimate the upper bound
#  of the credible interval to be equal to 0.25*df[, 957] + 0.75*df[, 958],
#  where each column of df has been sorted into ascending order.
#'
#' @param df The data frame with the parameter values.
#' @param percentage The percentage that we seek to find the credible interval
#'   for.
#'
#' @return
#' A data frame containing the lower and upper estimates of the credible 
#' intervals for each parameter.
#' @export
compute_cred_int <- function(df,
                             percentage) {
  upper_values <- rep(1, times = ncol(df))
  lower_values <- rep(1, times = ncol(df))

  row_nums <- nrow(df)

  print(paste0("row_nums: ", row_nums))

  upper_index <- (row_nums / 100) * (50 + percentage / 2)

  print(paste0("upper_index: ", upper_index))

  lower_index <- (row_nums / 100) * (50 - percentage / 2)

  print(paste0("lower_index: ", lower_index))

  # If upper_index and lower_index are not integers, we estimate the upper and
  # lower bounds of the interval.
  if (upper_index %% 1 != 0) {
    ufactor_floor <- upper_index - floor(upper_index)
    ufactor_ceiling <- ceiling(upper_index) - upper_index
    lfactor_floor <- lower_index - floor(lower_index)
    lfactor_ceiling <- ceiling(lower_index) - lower_index

    upper_floor <- floor(upper_index)
    upper_ceiling <- ceiling(upper_index)
    lower_floor <- floor(lower_index)
    lower_ceiling <- ceiling(lower_index)
  }

  for (i in 1:ncol(df)) {
    prelim_vec <- base::sort(as.vector(df[, i]))
    if (upper_index %% 1 == 0) {
      upper_values[i] <- prelim_vec[upper_index]
      lower_values[i] <- prelim_vec[lower_index]
    } else {
      upper_values[i] <- ufactor_ceiling * prelim_vec[upper_floor] +
        ufactor_floor * prelim_vec[upper_ceiling]
      lower_values[i] <- lfactor_ceiling * prelim_vec[lower_floor] +
        lfactor_floor * prelim_vec[lower_ceiling]
    }
  }

  return_df <- data.frame(
    "Lower credible estimates" = lower_values,
    "Upper credible estimates" = upper_values
  )

  rownames(return_df) <- colnames(df)

  return(return_df)
}