#' Find parameters for the normal priors of the transformed parameters
#'
#' @description
#' We calculate the means and standard deviations for the prior distributions
#' of the fits. In this function, we do this by assigning a lower and higher 
#' value associated with each parameter and assigning a cumulative probability 
#' to those two numbers. We then use the quantile function to determine the 
#' values of the parameters for each prior.
#'
#' @param smaller_values A vector of the smaller values for each parameter.
#' @param larger_values A vector of the larger values for each parameter.
#' @param lower_cd The cumulative probability attained when the
#'   parameter is equal to \code{lower_param_value}.
#' @param higher_cd The cumulative probability attained when the
#'   parameter is equal to \code{higher_param_value}.
#'
#' @return A list with the means and standard deviations of the normal prior for 
#'   each of the transformed parameters. These values are for
#'   the log-normal distributions of the untransformed parameters.
#' @export
#'
#' @examples
#' compute_prior_params(
#'   c(0.01, 100, 0.0001, 0.02), 
#'   c(20, 8000, 1, 1), 
#'   0.01, 
#'   0.99
#' )
compute_prior_params <- function(smaller_values,
                                 larger_values,
                                 lower_cd,
                                 higher_cd) {
  vec_sds <- vector()
  vec_means <- vector()

  numerator <- log(larger_values) - log(smaller_values)
  denominator <- sqrt(2) * (pracma::erfinv(2 * higher_cd - 1) -
    pracma::erfinv(2 * lower_cd - 1))
  vec_sds <- numerator / denominator
  vec_means <- log(larger_values) - sqrt(2) * vec_sds *
    pracma::erfinv(2 * higher_cd - 1)

  return_list <- list(vec_means, vec_sds)
  return_list
}
