#' Calculate the logarithms of model parameters 
#'
#' @param true_params A vector of, in order, (1-phi) mu, phi E, a-1, c-a, b-c.
#'
#' @return A vector with the logarithms of the model parameters.
#' @export
#'
#' @examples
#' transf_true_params_gt(c(5, 600, 0.02, 0.08, 0.1))
transf_true_params_gt <- function(true_params) {
  nu  <- log(true_params[1])
  eta <- log(true_params[2])
  tau <- log(true_params[3])
  xi  <- log(true_params[4])
  chi <- log(true_params[5])
  
  c(nu, eta, tau, xi, chi)
}

#' Calculate the model parameters from their logarithms
#'
#' @description
#' This function takes the logarithms of the five model parameters and creates
#' the following eight-element vector:
#' \itemize{
#' \item elements 1-5 contain the model parameters.
#' \item element 6 is the minimum recruitment stretch, or fibril length, a.
#' \item element 7 is the modal recruitment stretch, c.
#' \item element 8 is the maximum recruitment stretch, b.
#' }
#'
#' @param log_params A vector of, in order, nu, eta, tau, xi, chi.
#'
#' @return A vector containing eight elements that correspond to the five 
#'   fit parameters used in the RWM algorithm and the three distribution 
#'   parameters. (See the description above for more details.)
#' @export
#'
#' @examples
#' transf_log_params_gt(c(1.61, 6.40, -3.91, -2.53, -2))
transf_log_params_gt <- function(log_params) {
  shear_mod <- exp(log_params[1])
  youngs_mod <- exp(log_params[2])
  am1 <- exp(log_params[3])
  cma <- exp(log_params[4])
  bmc <- exp(log_params[5])
  a <- am1 + 1
  c <- a + cma
  b <- c + bmc
  
  c(shear_mod, youngs_mod, am1, cma, bmc, a, c, b)
}
