#' Plot mean stresses and a confidence band around the means (gen. tri. dist.)
#'
#' This function first takes a randomly selected subset of positions from the
#' Markov chains, with the size of the subset determined by the user. It then
#' calculates the mean and variance of the stresses, at each stretch given in
#' the experimental data, predicted by the chosen parameter vectors. The
#' function then plots the mean and a confidence band corresponding to a
#' particular multiple of the variance around the mean values. The experimental
#' stress-strain values are also plotted. Finally, the function zooms in on a
#' particular part of the x-axis to make the various elements of the plot more
#' visible for a section of the stress-strain curve.
#'
#' @param param_values_df Matrix of parameter values found by the Markov chains.
#' @param param_vec_num Number of parameter vectors, i.e. rows of
#'   \code{param_values_df}, that are selected.
#' @param cols_sampled Columns of \code{param_values_df} that are selected.
#' @param sigma_num How many sigma away from the mean does the confidence band
#'   extend.
#' @param strain_expt Vector of experimentally observed stretches.
#' @param stress_expt Vector of experimentally observed stresses.
#' @param save_title Title that the plots are saved under.
#' @param x_label Label along the x-axis of the plot.
#' @param y_label Label along the y-axis of the plot.
#' @param image_type String containing the image format that the plot is saved
#'   under, e.g. .jpg or .pdf.
#' @param pic_width Width of the plot.
#' @param aspect_ratio Aspect ratio of the plot.
#' @param pic_units String containing the units that the picture dimensions
#'   are calculated in, e.g. cm.
#' @param zoom_x_min Minimum stretch that we zoom-in on.
#' @param zoom_x_max Maximum stretch that we zoom-in on.
#' @param log_params A Boolean on whether \code{param_values_df} contains the
#'   transformed parameter values or not.
#'
#' @return Void
#'
#' @export
plot_MACB_gt <- function(param_values_df,
                         param_vec_num,
                         cols_sampled,
                         sigma_num,
                         strain_expt,
                         stress_expt,
                         save_title,
                         x_label,
                         y_label,
                         image_type,
                         pic_width,
                         aspect_ratio,
                         pic_units = "cm",
                         zoom_x_min = 1,
                         zoom_x_max = max(strain_expt),
                         log_params = FALSE) {
  strain <- means <- lower <- upper <- expdat <- NULL
  
  sample_vec <- seq(from = 1, to = nrow(param_values_df))

  vecs_subset <- sample(sample_vec, param_vec_num, replace = FALSE)

  prelim_mat <- param_values_df[vecs_subset, cols_sampled]

  if (log_params == FALSE) {
    prelim_mat <- log(prelim_mat)
  }

  stress_mat <- matrix(NA,
    nrow = length(vecs_subset),
    ncol = length(strain_expt)
  )

  # We plot the stress-strain curve for each of the parameter vectors that we
  # selected in order to construct the means and confidence band for the plot.
  for (i in 1:length(vecs_subset)) {
    stress_mat[i, ] <- calculate_gt_tot_1pkstress(
      prelim_mat[i, 1],
      prelim_mat[i, 2],
      prelim_mat[i, 3],
      prelim_mat[i, 4],
      prelim_mat[i, 5],
      strain_expt
    )
  }

  mean_stresses <- rep(0, length = length(strain_expt))
  sd_stresses <- rep(0, length = length(strain_expt))

  mean_stresses[1] <- 0.0
  sd_stresses[1] <- 0.0

  for (i in 2:length(strain_expt)) {
    mean_stresses[i] <- mean(stress_mat[, i])
    sd_stresses[i] <- sd(stress_mat[, i])
  }

  lb_stresses <- mean_stresses - sigma_num * sd_stresses
  ub_stresses <- mean_stresses + sigma_num * sd_stresses

  plot_df <- data.frame(
    strain = strain_expt,
    expdat = stress_expt,
    means = mean_stresses,
    lower = lb_stresses,
    upper = ub_stresses
  )

  p1 <- ggplot2::ggplot(data = plot_df) +
    ggplot2::geom_line(ggplot2::aes(
      x = strain,
      y = means
    ),
    colour = "#000000"
    ) +
    ggplot2::geom_ribbon(ggplot2::aes(
      ymin = lower,
      ymax = upper,
      x = strain
    ),
    alpha = 0.6,
    colour = "#004488",
    fill = "#004488"
    ) +
    ggplot2::theme_bw() +
    ggplot2::labs(
      x = parse(text = x_label),
      y = parse(text = y_label)
    ) +
    ggplot2::geom_point(ggplot2::aes(
      x = strain,
      y = expdat
    ),
    colour = "#DDAA33",
    alpha = 0.6,
    size = 1
    ) +
    ggplot2::theme(
      text = ggplot2::element_text(family = "LM Roman 10"),
      axis.title.x = ggplot2::element_text(size = 20, family = "LM Roman 10"),
      axis.title.y = ggplot2::element_text(size = 20, family = "LM Roman 10"),
      axis.text.x = ggplot2::element_text(size = 12),
      axis.text.y = ggplot2::element_text(size = 12)
    )

  pic_height <- pic_width / aspect_ratio

  # If we have specified a range of stretch that we wish to zoom in on, then
  # we zoom in on that section. Zooming-in on a section of the plot may be
  # necessary if the confidence band around the mean points is small.
  if (zoom_x_min > min(strain_expt) || zoom_x_max < max(strain_expt)) {
    if (zoom_x_min > min(strain_expt)) {
      zoom_min_index <- max(which(strain_expt < zoom_x_min))
      zoom_y_min <- lb_stresses[zoom_min_index]
    } else {
      zoom_y_min <- min(lb_stresses)
    }

    if (zoom_x_max < max(strain_expt)) {
      zoom_max_index <- min(which(strain_expt > zoom_x_max))
      zoom_y_max <- ub_stresses[zoom_max_index]
    } else {
      zoom_y_max <- max(ub_stresses)
    }

    p2 <- p1 + ggforce::facet_zoom(
      xlim = c(zoom_x_min, zoom_x_max),
      ylim = c(zoom_y_min, zoom_y_max),
      zoom.size = 2.0,
      show.area = TRUE,
      shrink = TRUE,
      split = FALSE,
      horizontal = FALSE
    )

    save_name <- paste0(save_title, "_noZoom.", image_type)
    save_name_zoom <- paste0(save_title, "_zoomedIn.", image_type)

    ggplot2::ggsave(save_name,
      plot = p1, width = pic_width,
      height = pic_height, units = "cm", dpi = 300
    )
    ggplot2::ggsave(save_name_zoom,
      plot = p2, width = pic_width,
      height = pic_height, units = "cm", dpi = 300
    )
  } else {
    save_name <- paste0(save_title, "_noZoom.", image_type)
    ggplot2::ggsave(save_name,
      plot = p1, width = pic_width, height = pic_height,
      units = pic_units, dpi = 300
    )
  }
}


#' Plot marginals and 2D contours of model parameters (general tri. dist.)
#'
#' This function takes a data frame of parameter values and creates a ggplot
#' matrix that contains marginal posteriors of the parameters, and contour
#' plots and correlations between the parameters with one another. The function
#' also changes the limits of the marginal distributions and contour plots in
#' order to make the distribution and contour lines, respectively, more easily
#' observable.
#'
#' @param df Data frame of parameter values.
#' @param vec_rows Vector with the first and last row of \code{df} to use to
#'   estimate marginal posteriors and 2D contours.
#' @param vec_cols Columns of \code{df} that are used.
#' @param pos_values_pc A float that defines the percentage of parameter values
#'   that are shown on the marginal posteriors. E.g. 99 would indicate that we
#'   zoom-in on the marginal posterior, with the limits given by the values at
#'   0.5% and 99.5% cumulative distribution. Note that all values in \code{df}
#'   are used to calculate the marginals and 2D contour plots.
#' @param cov_values_pc A float that defines which values from the
#'   bottom and top of the cumulative distribution are used to define the
#'   visible limits of the y-axis of the contour plots.
#' @param dps_vec Vector of decimal places that ticks are given to for each
#'   parameter.
#' @param parsed_labels_vec A vector of strings corresponding to the labels
#'   of the strips on the ggplot matrix. Note that these strings are parsed in
#'   the function.
#' @param x_strip_font_size The size of the font for the strips along the
#'   x-axis.
#' @param y_strip_font_size The size of the font for the strips along the
#'   y-axis.
#' @param pic_width Width of the plot.
#' @param aspect_ratio Aspect ratio of the plot.
#' @param pic_units String containing the units that the picture dimensions
#'   are calculated in, e.g. cm.
#' @param save_plot A Boolean that determines whether the ggplot matrix is saved
#'   or not.
#' @param save_title String containing the title that the plot matrix
#'   is saved under.
#' @param image_type String containing the image format that the plot is saved
#'   under, e.g. jpg or pdf.
#'
#' @return A square ggplot matrix object that contains marginal posteriors along
#'   the main diagonal, 2D contour plots between the parameters with one
#'   another on the lower triangle of the matrix, and correlations between the
#'   parameters with one another on the upper triangle of the matrix.
#'
#' @export
plot_corr_gt <- function(df,
                         vec_rows,
                         vec_cols,
                         pos_values_pc,
                         cov_values_pc,
                         dps_vec,
                         parsed_labels_vec,
                         x_strip_font_size,
                         y_strip_font_size,
                         pic_width = 17.4,
                         aspect_ratio = 1.36,
                         pic_units = "cm",
                         save_plot = TRUE,
                         save_title = NA,
                         image_type = NA) {
  m <- min(vec_rows)
  n <- max(vec_rows)

  p1 <- GGally::ggpairs(df[m:n, vec_cols],
    diag = list(continuous = "densityDiag", n = 8192, bw = "nrd"),
    lower = list(continuous = "density"),
    columnLabels = parsed_labels_vec,
    labeller = "label_parsed",
    upper = list(continuous = GGally::wrap("cor", family = "LM Roman 10"))
  )

  # We identify the new limits for the marginal posteriors and contour plots to
  # make the forms of the marginals and contour lines more easily viewable
  rows_num <- nrow(df[m:n, ])

  prelim_pos_lower <- (rows_num / 100) * (50 - pos_values_pc / 2)
  prelim_pos_upper <- (rows_num / 100) * (50 + pos_values_pc / 2)
  lower_pos_index <- floor(prelim_pos_lower)
  upper_pos_index <- ceiling(prelim_pos_upper)

  pos_lim_mat <- matrix(0, nrow = 2, ncol = length(vec_cols))

  prelim_cov_lower <- (rows_num / 100) * (50 - cov_values_pc / 2)
  prelim_cov_upper <- (rows_num / 100) * (50 + cov_values_pc / 2)
  lower_cov_index <- floor(prelim_cov_lower)
  upper_cov_index <- ceiling(prelim_cov_upper)

  cov_lim_mat <- matrix(0, nrow = 2, ncol = length(vec_cols))

  for (i in 1:length(vec_cols)) {
    prelim_values_vec <- sort(df[m:n, vec_cols[i]])

    lower_pos_lim <- prelim_values_vec[lower_pos_index]
    upper_pos_lim <- prelim_values_vec[upper_pos_index]

    pos_lim_mat[, i] <- c(lower_pos_lim, upper_pos_lim)

    lower_cov_lim <- prelim_values_vec[lower_cov_index]
    upper_cov_lim <- prelim_values_vec[upper_cov_index]

    cov_lim_mat[, i] <- c(lower_cov_lim, upper_cov_lim)
  }

  # We change the text of the plots so that the font matches LaTeX fonts
  for (i in 1:length(vec_cols)) {
    p1[i, i] <- p1[i, i] + ggplot2::coord_cartesian(xlim = c(
      pos_lim_mat[1, i],
      pos_lim_mat[2, i]
    )) +
      ggplot2::theme(
        text = ggplot2::element_text(family = "LM Roman 10"),
        axis.text.x = ggplot2::element_text(
          size = 12,
          family = "LM Roman 10"
        ),
        legend.position = "none",
        panel.grid.major = ggplot2::element_blank(),
        panel.grid.minor = ggplot2::element_blank()
      )
  }

  for (i in 2:length(vec_cols)) {
    j <- 1
    while (j < i) {
      p1[i, j] <- p1[i, j] + ggplot2::coord_cartesian(
        xlim = c(
          pos_lim_mat[1, j],
          pos_lim_mat[2, j]
        ),
        ylim = c(
          cov_lim_mat[1, i],
          cov_lim_mat[2, i]
        )
      ) +
        ggplot2::theme(
          text = ggplot2::element_text(
            family = "LM Roman 10",
            size = 12
          ),
          legend.position = "none",
          panel.grid.major = ggplot2::element_blank(),
          panel.grid.minor = ggplot2::element_blank()
        )
      j <- j + 1
    }
  }

  # We change the ticks in the plot matrix to avoid ticks overlapping
  for (i in 1:length(vec_cols)) {
    diff_quart <- (pos_lim_mat[2, i] - pos_lim_mat[1, i]) / 5
    low_tick <- pos_lim_mat[1, i] + diff_quart
    high_tick <- pos_lim_mat[2, i] - diff_quart

    low_tick <- round(low_tick, digits = dps_vec[i])
    high_tick <- round(high_tick, digits = dps_vec[i])

    ticks_vec <- c(low_tick, high_tick)

    p1[5, i] <- p1[5, i] + ggplot2::scale_x_continuous(breaks = ticks_vec) +
      ggplot2::theme(axis.text.x = ggplot2::element_text(
        size = 12,
        family = "LM Roman 10"
      ))
  }

  for (i in 1:length(vec_cols)) {
    if (i > 1) {
      diff_quart2 <- (cov_lim_mat[2, i] - cov_lim_mat[1, i]) / 5
      low_tick2 <- cov_lim_mat[1, i] + diff_quart2
      high_tick2 <- cov_lim_mat[2, i] - diff_quart2

      low_tick2 <- round(low_tick2, digits = dps_vec[i])
      high_tick2 <- round(high_tick2, digits = dps_vec[i])

      ticks_vec2 <- c(low_tick2, high_tick2)

      p1[i, 1] <- p1[i, 1] + ggplot2::scale_y_continuous(breaks = ticks_vec2) +
        ggplot2::theme(axis.text.y = ggplot2::element_text(
          size = 12,
          family = "LM Roman 10"
        ))
    }
  }

  p1[1, 1] <- p1[1, 1] +
    ggplot2::theme(axis.text.y = ggplot2::element_text(
      size = 12,
      family = "LM Roman 10"
    ))

  p1 <- p1 +
    ggplot2::theme(
      strip.text.x = ggplot2::element_text(
        size = x_strip_font_size,
        family = "LM Roman 10"
      ),
      strip.text.y = ggplot2::element_text(
        size = y_strip_font_size,
        family = "LM Roman 10"
      )
    )

  if (save_plot == TRUE) {
    pic_height <- pic_width / aspect_ratio

    ggsave_title <- paste(save_title, ".", image_type, sep = "")

    ggplot2::ggsave(ggsave_title,
      plot = p1, dpi = 300, width = pic_width,
      height = pic_height, units = pic_units, cairo_pdf
    )
  }
  return(p1)
}

#' Plot marginals and 2D contours from fit to synthetic data (gen. tri. dist.)
#'
#' This function takes a data frame of parameter values and creates a ggplot
#' matrix that contains marginal posteriors of the parameters, and contour
#' plots and correlations between the parameters with one another. The
#' parameter values used to create the synthetic data are shown on the plots as
#' a red line on the marginal posteriors and as a black dot on the 2D contour
#' plots. The function also changes the limits of the marginal distributions and
#' contour plots in order to make the distribution and contour lines,
#' respectively, more easily observable.
#'
#' @param df Data frame of parameter values.
#' @param vec_rows Vector with the first and last row of \code{df} to use to
#'   estimate marginal posteriors and 2D contours.
#' @param vec_cols Columns of \code{df} that are used.
#' @param pos_values_pc A float that defines the percentage of parameter values
#'   that are shown on the marginal posteriors. E.g. 99 would indicate that we
#'   zoom-in on the marginal posterior, with the limits given by the values at
#'   0.5% and 99.5% cumulative distribution. Note that all values in \code{df}
#'   are used to calculate the marginals and 2D contour plots.
#' @param cov_values_pc A float that determines which values from the
#'   bottom and top of the cumulative distribution are used to define the
#'   visible limits of the y-axis of the contour plots.
#' @param dps_vec Vector of decimal places that ticks are given to for each
#'   parameter.
#' @param vec_intercepts A vector containing the values used to create the
#'   synthetic stress-strain data.
#' @param parsed_labels_vec A vector of strings corresponding to the labels
#'   of the strips on the ggplot matrix. Note that these strings are parsed in
#'   the function.
#' @param x_strip_font_size The size of the font for the strips along the
#'   x-axis.
#' @param y_strip_font_size The size of the font for the strips along the
#'   y-axis.
#' @param pic_width Width of the plot.
#' @param aspect_ratio Aspect ratio of the plot.
#' @param pic_units String containing the units that the picture dimensions
#'   are calculated in, e.g. cm.
#' @param save_plot A Boolean that determines whether the ggplot matrix is saved
#'   or not.
#' @param save_title Title that the plots are saved under.
#' @param image_type String containing the image format that the plot is saved
#'   under, e.g. .jpg or .pdf.
#'
#' @return A square ggplot matrix object that contains marginal posteriors along
#'   the main diagonal, 2D contour plots between the parameters with one
#'   another on the lower triangle of the matrix, and correlations between the
#'   parameters with one another on the upper triangle of the matrix.
#'
#' @export
plot_corr_synth_gt <- function(df,
                               vec_rows,
                               vec_cols,
                               pos_values_pc,
                               cov_values_pc,
                               dps_vec,
                               vec_intercepts,
                               parsed_labels_vec,
                               x_strip_font_size,
                               y_strip_font_size,
                               pic_width = 17.4,
                               aspect_ratio = 1.36,
                               pic_units = "cm",
                               save_plot = TRUE,
                               save_title = NA,
                               image_type = NA) {
  xpos <- ypos <- NULL
  
  m <- min(vec_rows)
  n <- max(vec_rows)

  p1 <- GGally::ggpairs(df[m:n, vec_cols],
    lower = list(continuous = "density"),
    columnLabels = parsed_labels_vec,
    labeller = "label_parsed",
    upper = list(continuous = GGally::wrap("cor", family = "LM Roman 10"))
  )

  # We identify the new limits for the marginal posteriors and contour plots to
  # make the forms of the marginals and contour lines more easily viewable
  rows_num <- nrow(df[m:n, ])

  prelim_pos_lower <- (rows_num / 100) * (50 - pos_values_pc / 2)
  prelim_pos_upper <- (rows_num / 100) * (50 + pos_values_pc / 2)
  lower_pos_index <- floor(prelim_pos_lower)
  upper_pos_index <- ceiling(prelim_pos_upper)

  pos_lim_mat <- matrix(0, nrow = 2, ncol = length(vec_cols))

  prelim_cov_lower <- (rows_num / 100) * (50 - cov_values_pc / 2)
  prelim_cov_upper <- (rows_num / 100) * (50 + cov_values_pc / 2)
  lower_cov_index <- floor(prelim_cov_lower)
  upper_cov_index <- ceiling(prelim_cov_upper)

  cov_lim_mat <- matrix(0, nrow = 2, ncol = length(vec_cols))

  for (i in 1:length(vec_cols)) {
    prelim_values_vec <- sort(df[m:n, vec_cols[i]])

    lower_pos_lim <- prelim_values_vec[lower_pos_index]
    upper_pos_lim <- prelim_values_vec[upper_pos_index]

    pos_lim_mat[, i] <- c(lower_pos_lim, upper_pos_lim)

    lower_cov_lim <- prelim_values_vec[lower_cov_index]
    upper_cov_lim <- prelim_values_vec[upper_cov_index]

    cov_lim_mat[, i] <- c(lower_cov_lim, upper_cov_lim)
  }

  # We change the text of the plots so that the font matches LaTeX fonts
  for (i in 1:length(vec_cols)) {
    p1[i, i] <- p1[i, i] + ggplot2::coord_cartesian(xlim = c(
      pos_lim_mat[1, i],
      pos_lim_mat[2, i]
    )) +
      ggplot2::geom_vline(
        xintercept = vec_intercepts[i],
        linetype = "dashed",
        colour = "#BB5566",
        size = 2
      ) +
      ggplot2::theme(
        text = ggplot2::element_text(family = "LM Roman 10"),
        legend.position = "none",
        panel.grid.major = ggplot2::element_blank(),
        panel.grid.minor = ggplot2::element_blank()
      )
  }

  for (i in 2:length(vec_cols)) {
    j <- 1
    while (j < i) {
      marker_df <- data.frame(
        "xpos" = vec_intercepts[j],
        "ypos" = vec_intercepts[i]
      )

      p1[i, j] <- p1[i, j] + ggplot2::coord_cartesian(
        xlim = c(
          pos_lim_mat[1, j],
          pos_lim_mat[2, j]
        ),
        ylim = c(
          cov_lim_mat[1, i],
          cov_lim_mat[2, i]
        )
      ) +
        ggplot2::geom_point(
          data = marker_df,
          ggplot2::aes(
            x = xpos,
            y = ypos
          ),
          colour = "#000000",
          size = 2
        ) +
        ggplot2::theme(
          text = ggplot2::element_text(family = "LM Roman 10"),
          legend.position = "none",
          panel.grid.major = ggplot2::element_blank(),
          panel.grid.minor = ggplot2::element_blank()
        )
      j <- j + 1
    }
  }

  # We change the ticks in the plot matrix to avoid ticks overlapping
  for (i in 1:length(vec_cols)) {
    diff_quart <- (pos_lim_mat[2, i] - pos_lim_mat[1, i]) / 5
    low_tick <- pos_lim_mat[1, i] + diff_quart
    high_tick <- pos_lim_mat[2, i] - diff_quart

    low_tick <- round(low_tick, digits = dps_vec[i])
    high_tick <- round(high_tick, digits = dps_vec[i])

    ticks_vec <- c(low_tick, high_tick)

    p1[5, i] <- p1[5, i] + ggplot2::scale_x_continuous(breaks = ticks_vec) +
      ggplot2::theme(axis.text.x = ggplot2::element_text(
        size = 12,
        family = "LM Roman 10"
      ))
  }

  for (i in 1:length(vec_cols)) {
    if (i > 1) {
      diff_quart2 <- (cov_lim_mat[2, i] - cov_lim_mat[1, i]) / 5
      low_tick2 <- cov_lim_mat[1, i] + diff_quart2
      high_tick2 <- cov_lim_mat[2, i] - diff_quart2

      low_tick2 <- round(low_tick2, digits = dps_vec[i])
      high_tick2 <- round(high_tick2, digits = dps_vec[i])

      ticks_vec2 <- c(low_tick2, high_tick2)

      p1[i, 1] <- p1[i, 1] + ggplot2::scale_y_continuous(breaks = ticks_vec2) +
        ggplot2::theme(axis.text.y = ggplot2::element_text(
          size = 12,
          family = "LM Roman 10"
        ))
    }
  }

  p1[1, 1] <- p1[1, 1] +
    ggplot2::theme(axis.text.y = ggplot2::element_text(
      size = 12,
      family = "LM Roman 10"
    ))

  p1 <- p1 +
    ggplot2::theme(
      strip.text.x = ggplot2::element_text(
        size = x_strip_font_size,
        family = "LM Roman 10"
      ),
      strip.text.y = ggplot2::element_text(
        size = y_strip_font_size,
        family = "LM Roman 10"
      )
    )

  if (save_plot == TRUE) {
    pic_height <- pic_width / aspect_ratio

    ggsave_title <- paste(save_title, ".", image_type, sep = "")

    ggplot2::ggsave(ggsave_title,
      plot = p1, dpi = 300, width = pic_width,
      height = pic_height, units = pic_units, cairo_pdf
    )
  }
  return(p1)
}


#' Plot histograms that show the relationship of the distribution parameters
#'
#' This function first takes columns from a data frame that correspond to the 
#' three parameters of the general triangular distribution used to model the 
#' distribution of the recruitment stretch, of fibril length, in the tissue: a, 
#' the minimum recruitment stretch; b, the maximum recruitment stretch; and c,
#' the modal recruitment stretch. Then the function creates two histograms, one 
#' which measures the value of 2c - b - a and one which measures the value of 
#' 2c - b - a / (b-a). We refer to these as the 'unnormalised' and 'normalised'
#' histograms, respectively, in the function as the values of the latter
#' quantity range between -1 and 1. In these histograms the y-axis plots the 
#' count.
#'
#' @param df Data frame of parameter values.
#' @param min_rs_col Column of the data frame that contains the values of a.
#' @param modal_rs_col Column of the data frame that contains the values of c.
#' @param max_rs_col Column of the data frame that contains the values of b.
#' @param pic_width Width of both histograms.
#' @param aspect_ratio Aspect ratio of both histograms.
#' @param savetitle_unnorm The title given to the 'unnormalised' histogram
#'   when it is saved.
#' @param savetitle_norm The title given to the 'normalised' histogram
#'   when it is saved.
#' @param pic_units String containing the units that the picture dimensions
#'   are calculated in, e.g. cm.
#' @param binnum_unnorm The number of bins in the 'unnormalised' histogram.
#' @param binnum_norm The number bins in the 'normalised' histogram.
#' @param xlabel_unnorm The label of the x-axis in the 'unnormalised' histogram.
#' @param xlabel_norm The label of the x-axis in the 'normalised' histogram.
#' @param xticks A vector of ticks for the x-axis.
#' @param yticks A vector of ticks for the y-axis.
#' @param title_size The size, in pt, of the font that makes up the titles of
#'   the x and y axes.
#' @param image_type The type of file that the image is saved as.
#'   
#' @return A vector of the two histograms (that is, ggplot objects). The first
#'   element of the vector is the 'unnormalised' histogram and the second
#'   element of the vector is the 'normalised' histogram.
#'
#' @export
plot_distparam_hist <- function(df,
                                min_rs_col,
                                modal_rs_col,
                                max_rs_col,
                                pic_width,
                                aspect_ratio,
                                savetitle_unnorm,
                                savetitle_norm,
                                pic_units = "cm",
                                binnum_unnorm = 50,
                                binnum_norm = 50,
                                xlabel_unnorm = "Value",
                                xlabel_norm = "Value",
                                xticks = NULL,
                                yticks = NULL,
                                title_size = 12,
                                image_type = "pdf") {
  c1 <- min_rs_col
  c2 <- modal_rs_col
  c3 <- max_rs_col
  
  interim_vec1 <- 2*df[, c2] - df[, c3] - df[, c1]
  interim_vec2 <- interim_vec1/(trueparam_df[, c3] - trueparam_df[, c1])
  
  histogram_df <- data.frame("Unnormalised" = interim_vec1,
                             "Normalised" = interim_vec2)

  p1 <- ggplot2::ggplot(data = histogram_df, 
                        aes(x = Unnormalised)) +
    ggplot2::geom_histogram(
      colour = "#000000",
      fill = "#004488",
      bins = binnum_unnorm,
    ) +
    ggplot2::theme_bw() +
    ggplot2::labs(
      x = parse(text = xlabel_unnorm),
      y = "Count"
    ) +
    ggplot2::theme(
      text = ggplot2::element_text(family = "LM Roman 10"),
      axis.title.x = ggplot2::element_text(
        size = title_size,
        family = "LM Roman 10"
      ),
      axis.title.y = ggplot2::element_text(
        size = title_size,
        family = "LM Roman 10"
      ),
      axis.text.x = ggplot2::element_text(size = 12),
      axis.text.y = ggplot2::element_text(size = 12),
    )
  
  if(is.null(xticks) == FALSE){
    p1 <- p1 + ggplot2::scale_x_continuous(breaks = xticks)
  }
  
  if(is.null(yticks) == FALSE){
    p1 <- p1 + ggplot2::scale_y_continuous(breaks = yticks)
  }

  pic_height <- pic_width / aspect_ratio
  save_name1 <- paste0(savetitle_unnorm, ".", image_type)
  ggplot2::ggsave(save_name1,
    plot = p1, width = pic_width, height = pic_height,
    units = pic_units, dpi = 300
  )

  p2 <- ggplot2::ggplot(data = histogram_df, 
                        aes(x = Normalised)) +
    ggplot2::geom_histogram(
      colour = "#000000",
      fill = "#004488",
      bins = binnum_norm
    ) +
    ggplot2::theme_bw() +
    ggplot2::labs(
      x = parse(text = xlabel_norm),
      y = "Count"
    ) +
    ggplot2::theme(
      text = ggplot2::element_text(family = "LM Roman 10"),
      axis.title.x = ggplot2::element_text(
        size = title_size,
        family = "LM Roman 10"
      ),
      axis.title.y = ggplot2::element_text(
        size = title_size,
        family = "LM Roman 10"
      ),
      axis.text.x = ggplot2::element_text(size = 12),
      axis.text.y = ggplot2::element_text(size = 12),
    )
  
  if(is.null(xticks) == FALSE){
    p2 <- p2 + ggplot2::scale_x_continuous(breaks = xticks)
  }
  
  if(is.null(yticks) == FALSE){
    p2 <- p2 + ggplot2::scale_y_continuous(breaks = yticks)
  }

  save_name2 <- paste0(savetitle_norm, ".", image_type)
  ggplot2::ggsave(save_name2,
    plot = p2, width = pic_width, height = pic_height,
    units = pic_units, dpi = 300, cairo_pdf
  )

  return(c(p1, p2))
}


#' Plot density histograms of the relationship of the distribution parameters
#'
#' This function first takes columns from a data frame that correspond to the 
#' three parameters of the general triangular distribution used to model the 
#' distribution of the recruitment stretch, or fibril length, in the tissue: a, 
#' the minimum recruitment stretch; b, the maximum recruitment stretch; and c, 
#' the modal recruitment stretch. Then the function creates two histograms, one 
#' which measures the value of 2c - b - a and one which measures the value of 
#' 2c - b - a / (b-a). We refer to these as the 'unnormalised' and 'normalised'
#' histograms, respectively, in this function as the values of the latter
#' quantity range between -1 and 1. In these histograms the y-axis plots the 
#' probability density.
#'
#' @param df Data frame of parameter values.
#' @param min_rs_col Column of the data frame that contains the values of a.
#' @param modal_rs_col Column of the data frame that contains the values of c.
#' @param max_rs_col Column of the data frame that contains the values of b.
#' @param pic_width Width of both histograms.
#' @param aspect_ratio Aspect ratio of both histograms.
#' @param savetitle_unnorm The title given to the 'unnormalised' histogram
#'   when it is saved.
#' @param savetitle_norm The title given to the 'normalised' histogram
#'   when it is saved.
#' @param pic_units String containing the units that the picture dimensions
#'   are calculated in, e.g. cm.
#' @param binnum_unnorm The number of bins in the 'unnormalised' histogram.
#' @param binnum_norm The number bins in the 'normalised' histogram.
#' @param xlabel_unnorm The label of the x-axis in the 'unnormalised' histogram.
#' @param xlabel_norm The label of the x-axis in the 'normalised' histogram.
#' @param xticks A vector of ticks for the x-axis.
#' @param yticks A vector of ticks for the y-axis.
#' @param title_size The size, in pt, of the font that makes up the titles of
#'   the x and y axes.
#' @param image_type The type of file that the image is saved as.
#'   
#' @return A vector of the two histograms (that is, ggplot objects). The first
#'   element of the vector is the 'unnormalised' histogram and the second
#'   element of the vector is the 'normalised' histogram.
#'
#' @export
plot_distparam_hist_density <- function(df,
                                        min_rs_col,
                                        modal_rs_col,
                                        max_rs_col,
                                        pic_width,
                                        aspect_ratio,
                                        savetitle_unnorm,
                                        savetitle_norm,
                                        pic_units = "cm",
                                        binnum_unnorm = 50,
                                        binnum_norm = 50,
                                        xlabel_unnorm = "Value",
                                        xlabel_norm = "Value",
                                        xticks = NULL,
                                        yticks = NULL,
                                        title_size = 12,
                                        image_type = "pdf") {
  c1 <- min_rs_col
  c2 <- modal_rs_col
  c3 <- max_rs_col
  
  interim_vec1 <- 2*df[, c2] - df[, c3] - df[, c1]
  interim_vec2 <- interim_vec1/(trueparam_df[, c3] - trueparam_df[, c1])
  
  histogram_df <- data.frame("Unnormalised" = interim_vec1,
                             "Normalised" = interim_vec2)
  
  p1 <- ggplot2::ggplot(data = histogram_df, 
                        aes(x = Unnormalised, y=after_stat(density))) +
    ggplot2::geom_histogram(
      colour = "#000000",
      fill = "#004488",
      bins = binnum_unnorm,
    ) +
    ggplot2::theme_bw() +
    ggplot2::labs(
      x = parse(text = xlabel_unnorm),
      y = "Density"
    ) +
    ggplot2::theme(
      text = ggplot2::element_text(family = "LM Roman 10"),
      axis.title.x = ggplot2::element_text(
        size = title_size,
        family = "LM Roman 10"
      ),
      axis.title.y = ggplot2::element_text(
        size = title_size,
        family = "LM Roman 10"
      ),
      axis.text.x = ggplot2::element_text(size = 12),
      axis.text.y = ggplot2::element_text(size = 12),
    )
  
  if(is.null(xticks) == FALSE){
    p1 <- p1 + ggplot2::scale_x_continuous(breaks = xticks)
  }
  
  if(is.null(yticks) == FALSE){
    p1 <- p1 + ggplot2::scale_y_continuous(breaks = yticks)
  }
  
  pic_height <- pic_width / aspect_ratio
  save_name1 <- paste0(savetitle_unnorm, ".", image_type)
  ggplot2::ggsave(save_name1,
                  plot = p1, width = pic_width, height = pic_height,
                  units = pic_units, dpi = 300
  )
  
  p2 <- ggplot2::ggplot(data = histogram_df, 
                        aes(x = Normalised, y=after_stat(density))) +
    ggplot2::geom_histogram(
      colour = "#000000",
      fill = "#004488",
      bins = binnum_norm
    ) +
    ggplot2::theme_bw() +
    ggplot2::labs(
      x = parse(text = xlabel_norm),
      y = "Density"
    ) +
    ggplot2::theme(
      text = ggplot2::element_text(family = "LM Roman 10"),
      axis.title.x = ggplot2::element_text(
        size = title_size,
        family = "LM Roman 10"
      ),
      axis.title.y = ggplot2::element_text(
        size = title_size,
        family = "LM Roman 10"
      ),
      axis.text.x = ggplot2::element_text(size = 12),
      axis.text.y = ggplot2::element_text(size = 12),
    )
  
  if(is.null(xticks) == FALSE){
    p2 <- p2 + ggplot2::scale_x_continuous(breaks = xticks)
  }
  
  if(is.null(yticks) == FALSE){
    p2 <- p2 + ggplot2::scale_y_continuous(breaks = yticks)
  }
  
  save_name2 <- paste0(savetitle_norm, ".", image_type)
  ggplot2::ggsave(save_name2,
                  plot = p2, width = pic_width, height = pic_height,
                  units = pic_units, dpi = 300, cairo_pdf
  )
  
  return(c(p1, p2))
}
