#' First function for the stress on the collagen fibrils (general tri. dist.)
#'
#' @description
#' Function that calculates the contribution to the Cauchy stress from the
#' collagen fibrils, when fibril length is modelled using a general triangular
#' distribution, when the stretch is greater than a, the minimum recruitment
#' stretch, but less than c, the modal recruitment stretch.
#'
#' @param eta The natural logarithm of phi*E, where E is the Young's modulus
#'   of the collagen fibrils and phi is the collagen fibril volume fraction.
#' @param tau The natural logarithm of a-1, where a is the minimum collagen
#'   fibril length.
#' @param xi The natural logarithm of c-a.
#' @param chi The natural logarithm of b-c.
#' @param lambda The stretch acting on the tissue.
#'
#' @return
#' A number, corresponding to the Cauchy stress acting on the collagen
#' fibrils.
#' @export
#'
#' @examples
#' calculate_gt_fib_cstress_1(6, -3, -3, -3, 1.06)
calculate_gt_fib_cstress_1 <- function(eta,
                                       tau,
                                       xi,
                                       chi,
                                       lambda) {
  term1 <- lambda^2
  term2 <- -2 * (1 + exp(tau)) * lambda * log(lambda / (1 + exp(tau)))
  term3 <- -(1 + exp(tau))^2
  
  prelim <- (term1 + term2 + term3) / ((exp(xi) + exp(chi))*exp(xi))
  
  exp(eta)*prelim
}


#' Second function for the stress on the collagen fibrils (general tri. dist.)
#'
#' @description
#' Function that calculates the contribution to the Cauchy stress from the
#' collagen fibrils, when fibril length is modelled using a general triangular
#' distribution, when the stretch is greater than c, the modal recruitment
#' stretch, but less than b, the maximum recruitment stretch.
#'
#' @param eta The natural logarithm of phi*E, where E is the Young's modulus
#'   of the collagen fibrils and phi is the collagen fibril volume fraction.
#' @param tau The natural logarithm of a-1, where a is the minimum collagen
#'   fibril length.
#' @param xi The natural logarithm of c-a.
#' @param chi The natural logarithm of b-c.
#' @param lambda The stretch acting on the tissue.
#'
#' @return
#' A number, corresponding to the Cauchy stress acting on the collagen
#' fibrils.
#' @export
#'
#' @examples
#' calculate_gt_fib_cstress_2(6, -3, -3, -3, 1.11)
calculate_gt_fib_cstress_2 <- function(eta,
                                       tau,
                                       xi,
                                       chi,
                                       lambda) {
  term1 <- (1 + exp(tau) + exp(xi))^2 / (exp(xi) * exp(chi))
  term2 <- -(1 + exp(tau))^2 / ((exp(xi) + exp(chi)) * exp(xi))
  term3 <- (2 * (1 + exp(tau)) * lambda * log(1 + exp(tau))) / 
    ((exp(xi) + exp(chi)) * exp(xi))
  term4 <- -(2 * (1 + exp(tau) + exp(xi)) * lambda * 
               log(1 + exp(tau) + exp(xi))) / (exp(xi) * exp(chi))
  term5 <- -lambda^2 / ((exp(xi) + exp(chi)) * exp(chi))
  term6 <- (2 * (1 + exp(tau) + exp(xi) + exp(chi)) * lambda * log(lambda)) / 
    ((exp(xi) + exp(chi)) * exp(chi))
  
  exp(eta)*(term1 + term2 + term3 + term4 + term5 + term6)
}


#' Third function for the stress on the collagen fibrils (general tri. dist.)
#'
#' @description
#' Function that calculates the contribution to the Cauchy stress from the
#' collagen fibrils, when fibril length is modelled using a general triangular
#' distribution, when the stretch is greater than b, the maximum recruitment
#' stretch.
#'
#' @param eta The natural logarithm of phi*E, where E is the Young's modulus
#'   of the collagen fibrils and phi is the collagen fibril volume fraction.
#' @param tau The natural logarithm of a-1, where a is the minimum collagen
#'   fibril length.
#' @param xi The natural logarithm of c-a.
#' @param chi The natural logarithm of b-c.
#' @param lambda The stretch acting on the tissue.
#'
#' @return
#' A number, corresponding to the Cauchy stress acting on the collagen
#' fibrils.
#' @export
#'
#' @examples
#' calculate_gt_fib_cstress_3(6, -3, -3, -3, 1.19)
calculate_gt_fib_cstress_3 <- function(eta,
                                       tau,
                                       xi,
                                       chi,
                                       lambda) {
  term1 <- -1
  term2 <- (2 * (1 + exp(tau)) * lambda * log(1 + exp(tau))) / 
    ((exp(xi) + exp(chi)) * exp(xi))
  term3 <- (2 * (1 + exp(tau) + exp(xi) + exp(chi)) * lambda * 
              log(1 + exp(tau) + exp(xi) + exp(chi))) / 
    ((exp(xi) + exp(chi)) * exp(chi))
  term4 <- -(2 * (1 + exp(tau) + exp(xi)) * lambda * 
               log(1 + exp(tau) + exp(xi))) / 
    (exp(xi) * exp(chi))
  
  exp(eta)*(term1 + term2 + term3 + term4)
}


#' Computes collagen fibril stress for a set of stretches (general tri. dist.)
#'
#' @description
#' Function that determines which expression, when fibril length is modelled
#' using a general triangular distribution, must be evaluated to calculate
#' the Cauchy stress acting on the fibrils at a given stretch, and which
#' evaluates each function for a vector of stretch values.
#'
#' @param eta The natural logarithm of phi*E, where E is the Young's modulus
#'   of the collagen fibrils and phi is the collagen fibril volume fraction.
#' @param tau The natural logarithm of a-1, where a is the minimum length of 
#'   the collagen fibrils.
#' @param xi The natural logarithm of c-a, where c is the modal length of the
#'   collagen fibrils.
#' @param chi The natural logarithm of b-c.
#' @param lambda The stretch acting on the tissue.
#'
#' @return
#' A vector, corresponding to the Cauchy stress acting on the collagen
#' fibrils at each stretch.
#' @export
#'
#' @examples
#' calculate_gt_fib_cstress(6, -3, -3, -3, c(1.06, 1.11, 1.2, 1.25, 1.3))
calculate_gt_fib_cstress <- function(eta,
                                     tau,
                                     xi,
                                     chi,
                                     lambda) {
  a <- 1 + exp(tau)
  n <- a + exp(xi)
  b <- n + exp(chi)
  l <- lambda
  
  prelim <- ifelse(a > l,
                   0,
                   ifelse(a <= l & l <= n,
                          calculate_gt_fib_cstress_1(eta, 
                                                     tau, 
                                                     xi, 
                                                     chi, 
                                                     l),
                          ifelse(n < l & l <= b,
                                 calculate_gt_fib_cstress_2(eta, 
                                                            tau, 
                                                            xi, 
                                                            chi, 
                                                            l),
                                 calculate_gt_fib_cstress_3(eta, 
                                                            tau, 
                                                            xi, 
                                                            chi, 
                                                            l)
                          )
                   )
  )
  prelim
}

#' Computes the tissue Cauchy stress for a set of stretches (general tri. dist.)
#'
#' @param nu The natural logarithm of (1-phi)*mu, where mu is the shear modulus
#'   of the non-collagenous matrix (NCM) and (1-phi) is the volume fraction
#'   of the NCM in the tissue.
#' @param eta The natural logarithm of phi*E, where E is the Young's modulus
#'   of the collagen fibrils and phi is the collagen fibril volume fraction.
#' @param tau The natural logarithm of a-1, where a is the minimum collagen
#'   fibril length.
#' @param xi The natural logarithm of c-a, where c is the modal collagen fibril
#'   length.
#' @param chi The natural logarithm of b-c, where b is the maximum collagen
#'   fibril length.
#' @param lambda The vector of stretches acting on the tissue.
#'
#' @return
#' A vector, corresponding to the Cauchy stress acting on the tissue at each
#' stretch measured.
#' @export
#'
#' @examples
#' calculate_gt_tot_cstress(1, 6, -3, -3, -3, c(1.06, 1.11, 1.2, 1.25, 1.3))
calculate_gt_tot_cstress <- function(nu,
                                     eta,
                                     tau,
                                     xi,
                                     chi,
                                     lambda) {
  prelim <- calculate_ncm_cstress(nu, lambda) +
    calculate_gt_fib_cstress(eta, tau, xi, chi, lambda)
  
  prelim
}

#' Computes tissue engineering stress for a set of stretches (gen. tri. dist.)
#'
#' @param nu The natural logarithm of (1-phi)*mu, where mu is the shear modulus
#'   of the non-collagenous matrix (NCM) and (1-phi) is the volume fraction
#'   of the NCM in the tissue.
#' @param eta The natural logarithm of phi*E, where E is the Young's modulus
#'   of the collagen fibrils and phi is the collagen fibril volume fraction.
#' @param tau The natural logarithm of a-1, where a is the minimum collagen
#'   fibril length.
#' @param xi The natural logarithm of c-a, where c is the modal collagen fibril
#'   length.
#' @param chi The natural logarithm of b-c, where b is the maximum collagen
#'   fibril length.
#' @param lambda The vector of stretches acting on the tissue.
#'
#' @return
#' A vector, corresponding to the engineering stress acting on the collagen
#' fibrils at each stretch.
#' @export
#'
#' @examples
#' calculate_gt_tot_1pkstress(1, 6, -3, -3, -3, c(1.08, 1.1, 1.2, 1.25, 1.3))
calculate_gt_tot_1pkstress <- function(nu,
                                       eta,
                                       tau,
                                       xi,
                                       chi,
                                       lambda) {
  calculate_gt_tot_cstress(nu, eta, tau, xi, chi, lambda) / lambda
}
