#' Calculate logarithms of the model parameters
#'
#' @param true_params A vector of, in order, (1-phi)*mu, phi*E, a-1, b-a.
#'
#' @return A vector with the logarithms of the model parameters.
#' @export
#'
#' @examples
#' transform_true_params(c(5, 600, 0.02, 0.08))
transform_true_params <- function(true_params) {
  nu <- log(true_params[1])
  eta <- log(true_params[2])
  tau <- log(true_params[3])
  rho <- log(true_params[4])

  c(nu, eta, tau, rho)
}

#' Calculate the model parameters from their logarithms
#'
#' @description
#' This function takes the logarithms of the four model parameters and creates
#' the following six-element vector:
#' \itemize{
#' \item elements 1-4 contain the model parameters.
#' \item element 5 is the minimal recruitment stretch, a.
#' \item element 6 is the maximal recruitment stretch, b.
#' }
#'
#' @param log_params A vector of, in order, nu, eta, tau, rho.
#'
#' @return A vector containing six elements that correspond to the four 
#'   fit parameters used in the RWM algorithm and the second distribution 
#'   parameters. (See the description above for more details.)
#' @export
#'
#' @examples
#' transform_log_params(c(1.61, 6.40, -3.91, -2.53))
transform_log_params <- function(log_params) {
  shear_mod <- exp(log_params[1])
  youngs_mod <- exp(log_params[2])
  am1 <- exp(log_params[3])
  bma <- exp(log_params[4])
  a <- am1 + 1
  b <- a + bma

  c(shear_mod, youngs_mod, am1, bma, a, b)
}
