#' Plot mean stresses and a confidence band around the means
#'
#' This function first takes a randomly selected subset of positions from the
#' Markov chains, with the size of the subset determined by the user. It then
#' calculates the mean and variance of the stresses, at each stretch given in
#' the experimental data, predicted by the chosen parameter vectors. The
#' function then plots the mean and a confidence band corresponding to a
#' particular multiple of the variance around the mean values. The experimental
#' stress-strain values are also plotted. Finally, the function zooms in on a
#' particular part of the x-axis to make the various elements of the plot more
#' visible for a section of the stress-strain curve.
#'
#' @param param_values_df Matrix of parameter values found by the Markov chains.
#' @param param_vec_num Number of parameter vectors, i.e. rows of
#'   \code{param_values_df}, that are selected.
#' @param cols_sampled Columns of \code{param_values_df} that are selected.
#' @param sigma_num How many sigma away from the mean does the confidence band
#'   extend.
#' @param strain_expt Vector of experimentally observed stretches.
#' @param stress_expt Vector of experimentally observed stresses.
#' @param save_title Title that the plots are saved under.
#' @param x_label Label along the x-axis of the plot.
#' @param y_label Label along the y-axis of the plot.
#' @param image_type String containing the image format that the plot is saved
#'   under, e.g. .jpg or .pdf.
#' @param pic_width Width of the plot.
#' @param aspect_ratio Aspect ratio of the plot.
#' @param pic_units String containing the units that the picture dimensions
#'   are calculated in, e.g. cm.
#' @param zoom_x_min Minimum stretch that we zoom-in on.
#' @param zoom_x_max Maximum stretch that we zoom-in on.
#' @param log_params A Boolean on whether \code{param_values_df} contains the
#'   transformed parameter values or not.
#'
#' @return Void
#'
#' @export
plot_MACB <- function(param_values_df,
                      param_vec_num,
                      cols_sampled,
                      sigma_num,
                      strain_expt,
                      stress_expt,
                      save_title,
                      x_label,
                      y_label,
                      image_type,
                      pic_width,
                      aspect_ratio,
                      pic_units = "cm",
                      zoom_x_min = 1,
                      zoom_x_max = max(strain_expt),
                      log_params = FALSE) {
  strain <- means <- lower <- upper <- expdat <- NULL
  
  sample_vec <- seq(from = 1, to = nrow(param_values_df))

  vecs_subset <- sample(sample_vec, param_vec_num, replace = FALSE)

  prelim_mat <- param_values_df[vecs_subset, cols_sampled]

  if (log_params == FALSE) {
    prelim_mat <- log(prelim_mat)
  }

  stress_mat <- matrix(NA,
    nrow = length(vecs_subset),
    ncol = length(strain_expt)
  )

  # We plot the stress-strain curve for each of the parameter vectors that we
  # selected in order to construct the means and confidence band for the plot.
  for (i in 1:length(vecs_subset)) {
    stress_mat[i, ] <- calculate_st_tot_1pkstress(
      prelim_mat[i, 1],
      prelim_mat[i, 2],
      prelim_mat[i, 3],
      prelim_mat[i, 4],
      strain_expt
    )
  }

  mean_stresses <- rep(0, length = length(strain_expt))
  sd_stresses <- rep(0, length = length(strain_expt))

  mean_stresses[1] <- 0.0
  sd_stresses[1] <- 0.0

  for (i in 2:length(strain_expt)) {
    mean_stresses[i] <- mean(stress_mat[, i])
    sd_stresses[i] <- sd(stress_mat[, i])
  }

  lb_stresses <- mean_stresses - sigma_num * sd_stresses
  ub_stresses <- mean_stresses + sigma_num * sd_stresses

  plot_df <- data.frame(
    strain = strain_expt,
    expdat = stress_expt,
    means = mean_stresses,
    lower = lb_stresses,
    upper = ub_stresses
  )

  p1 <- ggplot2::ggplot(data = plot_df) +
    ggplot2::geom_line(ggplot2::aes(
      x = strain,
      y = means
    ),
    colour = "#000000"
    ) +
    ggplot2::geom_ribbon(ggplot2::aes(
      ymin = lower,
      ymax = upper,
      x = strain
    ),
    alpha = 0.6,
    colour = "#004488",
    fill = "#004488"
    ) +
    ggplot2::theme_bw() +
    ggplot2::labs(
      x = parse(text = x_label),
      y = parse(text = y_label)
    ) +
    ggplot2::geom_point(ggplot2::aes(
      x = strain,
      y = expdat
    ),
    colour = "#DDAA33",
    alpha = 0.6,
    size = 1
    ) +
    ggplot2::theme(
      text = ggplot2::element_text(family = "LM Roman 10"),
      axis.title.x = ggplot2::element_text(size = 14, family = "LM Roman 10"),
      axis.title.y = ggplot2::element_text(size = 14, family = "LM Roman 10"),
      axis.text.x = ggplot2::element_text(size = 10),
      axis.text.y = ggplot2::element_text(size = 10)
    )

  pic_height <- pic_width / aspect_ratio

  # If we have specified a range of stretch that we wish to zoom in on, then
  # we zoom in on that section. Zooming-in on a section of the plot may be
  # necessary if the confidence band around the mean points is small.
  if (zoom_x_min > min(strain_expt) || zoom_x_max < max(strain_expt)) {
    if (zoom_x_min > min(strain_expt)) {
      zoom_min_index <- max(which(strain_expt < zoom_x_min))
      zoom_y_min <- lb_stresses[zoom_min_index]
    } else {
      zoom_y_min <- min(lb_stresses)
    }

    if (zoom_x_max < max(strain_expt)) {
      zoom_max_index <- min(which(strain_expt > zoom_x_max))
      zoom_y_max <- ub_stresses[zoom_max_index]
    } else {
      zoom_y_max <- max(ub_stresses)
    }

    p2 <- p1 + ggforce::facet_zoom(
      xlim = c(zoom_x_min, zoom_x_max),
      ylim = c(zoom_y_min, zoom_y_max),
      zoom.size = 2.0,
      show.area = TRUE,
      shrink = TRUE,
      split = FALSE,
      horizontal = FALSE
    )

    save_name <- paste0(save_title, "_noZoom.", image_type)
    save_name_zoom <- paste0(save_title, "_zoomedIn.", image_type)

    ggplot2::ggsave(save_name,
      plot = p1, width = pic_width,
      height = pic_height, units = "cm", dpi = 300
    )
    ggplot2::ggsave(save_name_zoom,
      plot = p2, width = pic_width,
      height = pic_height, units = "cm", dpi = 300
    )
  } else {
    save_name <- paste0(save_title, "_noZoom.", image_type)
    ggplot2::ggsave(save_name,
      plot = p1, width = pic_width, height = pic_height,
      units = pic_units, dpi = 300
    )
  }
}


#' Plot marginals and 2D contour plots from a data frame of parameter values
#'
#' This function takes a data frame of parameter values and creates a ggplot
#' matrix that contains marginal posteriors of the parameters, and contour
#' plots and correlations between the parameters with one another. The function
#' also changes the limits of the marginal distributions and contour plots in
#' order to make the distribution and contour lines, respectively, more easily
#' observable.
#'
#' @param df Data frame of parameter values.
#' @param vec_rows Vector with the first and last row of \code{df} to use to
#'   estimate marginal posteriors and 2D contours.
#' @param vec_cols Columns of \code{df} that are used.
#' @param pos_values_pc A float that defines the percentage of parameter values
#'   that are shown on the marginal posteriors. E.g. 99 would indicate that we
#'   zoom-in on the marginal posterior, with the limits given by the values at
#'   0.5% and 99.5% cumulative distribution. Note that all values in \code{df}
#'   are used to calculate the marginals and 2D contour plots.
#' @param cov_values_pc A float that defines which values from the
#'   bottom and top of the cumulative distribution are used to define the
#'   visible limits of the y-axis of the contour plots.
#' @param dps_vec Vector of decimal places that ticks are given to for each
#'   parameter.
#' @param parsed_labels_vec A vector of strings corresponding to the labels
#'   of the strips on the ggplot matrix. Note that these strings are parsed in
#'   the function.
#' @param x_strip_font_size The size of the font for the strips along the
#'   x-axis.
#' @param y_strip_font_size The size of the font for the strips along the
#'   y-axis.
#' @param pic_width Width of the plot.
#' @param aspect_ratio Aspect ratio of the plot.
#' @param pic_units String containing the units that the picture dimensions
#'   are calculated in, e.g. cm.
#' @param save_plot A Boolean that determines whether the ggplot matrix is saved
#'   or not.
#' @param save_title String containing the title that the plot matrix
#'   is saved under.
#' @param image_type String containing the image format that the plot is saved
#'   under, e.g. jpg or pdf.
#'
#' @return A square ggplot matrix object that contains marginal posteriors along
#'   the main diagonal, 2D contour plots between the parameters with one
#'   another on the lower triangle of the matrix, and correlations between the
#'   parameters with one another on the upper triangle of the matrix.
#'
#' @export
plot_corr <- function(df,
                      vec_rows,
                      vec_cols,
                      pos_values_pc,
                      cov_values_pc,
                      dps_vec,
                      parsed_labels_vec,
                      x_strip_font_size,
                      y_strip_font_size,
                      pic_width = 17.4,
                      aspect_ratio = 1.36,
                      pic_units = "cm",
                      save_plot = TRUE,
                      save_title = NA,
                      image_type = NA) {
  m <- min(vec_rows)
  n <- max(vec_rows)

  p1 <- GGally::ggpairs(df[m:n, vec_cols],
    diag = list(continuous = "densityDiag", n = 8192, bw = "nrd"),
    lower = list(continuous = "density"),
    columnLabels = parsed_labels_vec,
    labeller = "label_parsed",
    upper = list(continuous = GGally::wrap("cor", family = "LM Roman 10"))
  )

  # We identify the new limits for the marginal posteriors and contour plots to
  # make the forms of the marginals and contour lines more easily viewable
  rows_num <- nrow(df[m:n, ])

  prelim_pos_lower <- (rows_num / 100) * (50 - pos_values_pc / 2)
  prelim_pos_upper <- (rows_num / 100) * (50 + pos_values_pc / 2)
  lower_pos_index <- floor(prelim_pos_lower)
  upper_pos_index <- ceiling(prelim_pos_upper)

  pos_lim_mat <- matrix(0, nrow = 2, ncol = length(vec_cols))

  prelim_cov_lower <- (rows_num / 100) * (50 - cov_values_pc / 2)
  prelim_cov_upper <- (rows_num / 100) * (50 + cov_values_pc / 2)
  lower_cov_index <- floor(prelim_cov_lower)
  upper_cov_index <- ceiling(prelim_cov_upper)

  cov_lim_mat <- matrix(0, nrow = 2, ncol = length(vec_cols))

  for (i in 1:length(vec_cols)) {
    prelim_values_vec <- sort(df[m:n, vec_cols[i]])

    lower_pos_lim <- prelim_values_vec[lower_pos_index]
    upper_pos_lim <- prelim_values_vec[upper_pos_index]

    pos_lim_mat[, i] <- c(lower_pos_lim, upper_pos_lim)

    lower_cov_lim <- prelim_values_vec[lower_cov_index]
    upper_cov_lim <- prelim_values_vec[upper_cov_index]

    cov_lim_mat[, i] <- c(lower_cov_lim, upper_cov_lim)
  }

  # We change the text of the plots so that the font matches LaTeX fonts
  for (i in 1:length(vec_cols)) {
    p1[i, i] <- p1[i, i] + ggplot2::coord_cartesian(xlim = c(
      pos_lim_mat[1, i],
      pos_lim_mat[2, i]
    )) +
      ggplot2::theme(
        text = ggplot2::element_text(family = "LM Roman 10"),
        axis.text.x = ggplot2::element_text(
          size = 10,
          family = "LM Roman 10"
        ),
        legend.position = "none",
        panel.grid.major = ggplot2::element_blank(),
        panel.grid.minor = ggplot2::element_blank()
      )
  }

  for (i in 2:length(vec_cols)) {
    j <- 1
    while (j < i) {
      p1[i, j] <- p1[i, j] + ggplot2::coord_cartesian(
        xlim = c(
          pos_lim_mat[1, j],
          pos_lim_mat[2, j]
        ),
        ylim = c(
          cov_lim_mat[1, i],
          cov_lim_mat[2, i]
        )
      ) +
        ggplot2::theme(
          text = ggplot2::element_text(
            family = "LM Roman 10",
            size = 10
          ),
          legend.position = "none",
          panel.grid.major = ggplot2::element_blank(),
          panel.grid.minor = ggplot2::element_blank()
        )
      j <- j + 1
    }
  }

  # We change the ticks in the plot matrix to avoid ticks overlapping
  for (i in 1:length(vec_cols)) {
    diff_quart <- (pos_lim_mat[2, i] - pos_lim_mat[1, i]) / 5
    low_tick <- pos_lim_mat[1, i] + diff_quart
    high_tick <- pos_lim_mat[2, i] - diff_quart

    low_tick <- round(low_tick, digits = dps_vec[i])
    high_tick <- round(high_tick, digits = dps_vec[i])

    ticks_vec <- c(low_tick, high_tick)

    p1[4, i] <- p1[4, i] + ggplot2::scale_x_continuous(breaks = ticks_vec) +
      ggplot2::theme(axis.text.x = ggplot2::element_text(
        size = 10,
        family = "LM Roman 10"
      ))
  }

  for (i in 1:length(vec_cols)) {
    if (i > 1) {
      diff_quart2 <- (cov_lim_mat[2, i] - cov_lim_mat[1, i]) / 5
      low_tick2 <- cov_lim_mat[1, i] + diff_quart2
      high_tick2 <- cov_lim_mat[2, i] - diff_quart2

      low_tick2 <- round(low_tick2, digits = dps_vec[i])
      high_tick2 <- round(high_tick2, digits = dps_vec[i])

      ticks_vec2 <- c(low_tick2, high_tick2)

      p1[i, 1] <- p1[i, 1] + ggplot2::scale_y_continuous(breaks = ticks_vec2) +
        ggplot2::theme(axis.text.y = ggplot2::element_text(
          size = 10,
          family = "LM Roman 10"
        ))
    }
  }

  p1[1, 1] <- p1[1, 1] +
    ggplot2::theme(axis.text.y = ggplot2::element_text(
      size = 10,
      family = "LM Roman 10"
    ))

  p1 <- p1 +
    ggplot2::theme(
      strip.text.x = ggplot2::element_text(
        size = x_strip_font_size,
        family = "LM Roman 10"
      ),
      strip.text.y = ggplot2::element_text(
        size = y_strip_font_size,
        family = "LM Roman 10"
      )
    )

  if (save_plot == TRUE) {
    pic_height <- pic_width / aspect_ratio

    ggsave_title <- paste(save_title, ".", image_type, sep = "")

    ggplot2::ggsave(ggsave_title,
      plot = p1, dpi = 300, width = pic_width,
      height = pic_height, units = pic_units, cairo_pdf
    )
  }
  return(p1)
}

#' Create ggplot matrix from parameter values found by fitting to synthetic data
#'
#' This function takes a data frame of parameter values and creates a ggplot
#' matrix that contains marginal posteriors of the parameters, and contour
#' plots and correlations between the parameters with one another. The
#' parameter values used to create the synthetic data are shown on the plots as
#' a red line on the marginal posteriors and as a black dot on the 2D contour
#' plots. The function also changes the limits of the marginal distributions and
#' contour plots in order to make the distribution and contour lines,
#' respectively, more easily observable.
#'
#' @param df Data frame of parameter values.
#' @param vec_rows Vector with the first and last row of \code{df} to use to
#'   estimate marginal posteriors and 2D contours.
#' @param vec_cols Columns of \code{df} that are used.
#' @param pos_values_pc A float that defines the percentage of parameter values
#'   that are shown on the marginal posteriors. E.g. 99 would indicate that we
#'   zoom-in on the marginal posterior, with the limits given by the values at
#'   0.5% and 99.5% cumulative distribution. Note that all values in \code{df}
#'   are used to calculate the marginals and 2D contour plots.
#' @param cov_values_pc A float that determines which values from the
#'   bottom and top of the cumulative distribution are used to define the
#'   visible limits of the y-axis of the contour plots.
#' @param dps_vec Vector of decimal places that ticks are given to for each
#'   parameter.
#' @param vec_intercepts A vector containing the values used to create the
#'   synthetic stress-strain data.
#' @param parsed_labels_vec A vector of strings corresponding to the labels
#'   of the strips on the ggplot matrix. Note that these strings are parsed in
#'   the function.
#' @param x_strip_font_size The size of the font for the strips along the
#'   x-axis.
#' @param y_strip_font_size The size of the font for the strips along the
#'   y-axis.
#' @param pic_width Width of the plot.
#' @param aspect_ratio Aspect ratio of the plot.
#' @param pic_units String containing the units that the picture dimensions
#'   are calculated in, e.g. cm.
#' @param save_plot A Boolean that determines whether the ggplot matrix is saved
#'   or not.
#' @param save_title Title that the plots are saved under.
#' @param image_type String containing the image format that the plot is saved
#'   under, e.g. .jpg or .pdf.
#'
#' @return A square ggplot matrix object that contains marginal posteriors along
#'   the main diagonal, 2D contour plots between the parameters with one
#'   another on the lower triangle of the matrix, and correlations between the
#'   parameters with one another on the upper triangle of the matrix.
#'
#' @export
plot_corr_synth <- function(df,
                            vec_rows,
                            vec_cols,
                            pos_values_pc,
                            cov_values_pc,
                            dps_vec,
                            vec_intercepts,
                            parsed_labels_vec,
                            x_strip_font_size,
                            y_strip_font_size,
                            pic_width = 17.4,
                            aspect_ratio = 1.36,
                            pic_units = "cm",
                            save_plot = TRUE,
                            save_title = NA,
                            image_type = NA) {
  xpos <- ypos <- NULL
  
  m <- min(vec_rows)
  n <- max(vec_rows)

  p1 <- GGally::ggpairs(df[m:n, vec_cols],
    lower = list(continuous = "density"),
    columnLabels = parsed_labels_vec,
    labeller = "label_parsed",
    upper = list(continuous = GGally::wrap("cor", family = "LM Roman 10"))
  )

  # We identify the new limits for the marginal posteriors and contour plots to
  # make the forms of the marginals and contour lines more easily viewable
  rows_num <- nrow(df[m:n, ])

  prelim_pos_lower <- (rows_num / 100) * (50 - pos_values_pc / 2)
  prelim_pos_upper <- (rows_num / 100) * (50 + pos_values_pc / 2)
  lower_pos_index <- floor(prelim_pos_lower)
  upper_pos_index <- ceiling(prelim_pos_upper)

  pos_lim_mat <- matrix(0, nrow = 2, ncol = length(vec_cols))

  prelim_cov_lower <- (rows_num / 100) * (50 - cov_values_pc / 2)
  prelim_cov_upper <- (rows_num / 100) * (50 + cov_values_pc / 2)
  lower_cov_index <- floor(prelim_cov_lower)
  upper_cov_index <- ceiling(prelim_cov_upper)

  cov_lim_mat <- matrix(0, nrow = 2, ncol = length(vec_cols))

  for (i in 1:length(vec_cols)) {
    prelim_values_vec <- sort(df[m:n, vec_cols[i]])

    lower_pos_lim <- prelim_values_vec[lower_pos_index]
    upper_pos_lim <- prelim_values_vec[upper_pos_index]

    pos_lim_mat[, i] <- c(lower_pos_lim, upper_pos_lim)

    lower_cov_lim <- prelim_values_vec[lower_cov_index]
    upper_cov_lim <- prelim_values_vec[upper_cov_index]

    cov_lim_mat[, i] <- c(lower_cov_lim, upper_cov_lim)
  }

  # We change the text of the plots so that the font matches LaTeX fonts
  for (i in 1:length(vec_cols)) {
    p1[i, i] <- p1[i, i] + ggplot2::coord_cartesian(xlim = c(
      pos_lim_mat[1, i],
      pos_lim_mat[2, i]
    )) +
      ggplot2::geom_vline(
        xintercept = vec_intercepts[i],
        linetype = "dashed",
        colour = "#BB5566",
        size = 2
      ) +
      ggplot2::theme(
        text = ggplot2::element_text(family = "LM Roman 10"),
        legend.position = "none",
        panel.grid.major = ggplot2::element_blank(),
        panel.grid.minor = ggplot2::element_blank()
      )
  }

  for (i in 2:length(vec_cols)) {
    j <- 1
    while (j < i) {
      marker_df <- data.frame(
        "xpos" = vec_intercepts[j],
        "ypos" = vec_intercepts[i]
      )

      p1[i, j] <- p1[i, j] + ggplot2::coord_cartesian(
        xlim = c(
          pos_lim_mat[1, j],
          pos_lim_mat[2, j]
        ),
        ylim = c(
          cov_lim_mat[1, i],
          cov_lim_mat[2, i]
        )
      ) +
        ggplot2::geom_point(
          data = marker_df,
          ggplot2::aes(
            x = xpos,
            y = ypos
          ),
          colour = "#000000",
          size = 2
        ) +
        ggplot2::theme(
          text = ggplot2::element_text(family = "LM Roman 10"),
          legend.position = "none",
          panel.grid.major = ggplot2::element_blank(),
          panel.grid.minor = ggplot2::element_blank()
        )
      j <- j + 1
    }
  }

  # We change the ticks in the plot matrix to avoid ticks overlapping
  for (i in 1:length(vec_cols)) {
    diff_quart <- (pos_lim_mat[2, i] - pos_lim_mat[1, i]) / 5
    low_tick <- pos_lim_mat[1, i] + diff_quart
    high_tick <- pos_lim_mat[2, i] - diff_quart

    low_tick <- round(low_tick, digits = dps_vec[i])
    high_tick <- round(high_tick, digits = dps_vec[i])

    ticks_vec <- c(low_tick, high_tick)

    p1[4, i] <- p1[4, i] + ggplot2::scale_x_continuous(breaks = ticks_vec) +
      ggplot2::theme(axis.text.x = ggplot2::element_text(
        size = 10,
        family = "LM Roman 10"
      ))
  }

  for (i in 1:length(vec_cols)) {
    if (i > 1) {
      diff_quart2 <- (cov_lim_mat[2, i] - cov_lim_mat[1, i]) / 5
      low_tick2 <- cov_lim_mat[1, i] + diff_quart2
      high_tick2 <- cov_lim_mat[2, i] - diff_quart2

      low_tick2 <- round(low_tick2, digits = dps_vec[i])
      high_tick2 <- round(high_tick2, digits = dps_vec[i])

      ticks_vec2 <- c(low_tick2, high_tick2)

      p1[i, 1] <- p1[i, 1] + ggplot2::scale_y_continuous(breaks = ticks_vec2) +
        ggplot2::theme(axis.text.y = ggplot2::element_text(
          size = 10,
          family = "LM Roman 10"
        ))
    }
  }

  p1[1, 1] <- p1[1, 1] +
    ggplot2::theme(axis.text.y = ggplot2::element_text(
      size = 10,
      family = "LM Roman 10"
    ))

  p1 <- p1 +
    ggplot2::theme(
      strip.text.x = ggplot2::element_text(
        size = x_strip_font_size,
        family = "LM Roman 10"
      ),
      strip.text.y = ggplot2::element_text(
        size = y_strip_font_size,
        family = "LM Roman 10"
      )
    )

  if (save_plot == TRUE) {
    pic_height <- pic_width / aspect_ratio

    ggsave_title <- paste(save_title, ".", image_type, sep = "")

    ggplot2::ggsave(ggsave_title,
      plot = p1, dpi = 300, width = pic_width,
      height = pic_height, units = pic_units, cairo_pdf
    )
  }
  return(p1)
}


#' Input new ticks for individual plots within a ggplot matrix object
#'
#' @param plot_matrix Ggplot matrix object.
#' @param coords_mat Matrix where each row corresponds to the coordinates of
#'   a particular within the matrix whose ticks will be changed.
#' @param new_ticks_mat Matrix with two columns. Each row represents a new set
#'   of two ticks to be included on the figure.
#' @param x_axis_vec A vector of Boolean values determining whether the new
#'   ticks are added to the x-axis, if TRUE, or the y-axis.
#' @param pic_width The dimensionless width of the new ggplot matrix object
#'   when saving.
#' @param aspect_ratio The aspect ratio of the new ggplot matrix object when
#'   saving.
#' @param pic_units The units to save the new ggplot matrix object in. E.g. cm.
#'   The default is cm.
#' @param save_plot A Boolean on whether the ggplot matrix is saved or not.
#'   Default is FALSE.
#' @param save_title A string containing the name that the ggplot matrix is
#'   saved under.
#' @param image_type A string containing the file type to save new ggplot
#'   matrix as. E.g. jpg or pdf.
#'
#' @return A square ggplot matrix object.
#'
#' @export
change_ticks_ggmat <- function(plot_matrix,
                               coords_mat,
                               new_ticks_mat,
                               x_axis_vec,
                               pic_width = 17.4,
                               aspect_ratio = 1.36,
                               pic_units = "cm",
                               save_plot = FALSE,
                               save_title = NA,
                               image_type = NA) {
  for (h in 1:nrow(coords_mat)) {
    i <- coords_mat[h, 1]
    j <- coords_mat[h, 2]

    if (x_axis_vec[h] == TRUE) {
      plot_matrix[i, j] <- plot_matrix[i, j] +
        ggplot2::scale_x_continuous(breaks = new_ticks_mat[h, ])
    } else if (x_axis_vec[h] == FALSE) {
      plot_matrix[i, j] <- plot_matrix[i, j] +
        ggplot2::scale_y_continuous(breaks = new_ticks_mat[h, ])
    }
  }
  if (save_plot == TRUE) {
    pic_height <- pic_width / aspect_ratio

    file_save_name <- paste0(save_title, ".", image_type)
    ggplot2::ggsave(file_save_name,
      plot = plot_matrix,
      dpi = 300,
      height = pic_height,
      width = pic_width,
      units = pic_units,
      cairo_pdf
    )
  }
  return(plot_matrix)
}


#' Input new limits for individual figures within a ggplot matrix object.
#'
#' @param plot_matrix Ggplot matrix object.
#' @param cols_changed_vec Vector of columns of \code{plot_matrix} where the
#'   limits of figures will be changed.
#' @param x_lims_mat A matrix with two columns. The \emph{i}th row contains a
#'   new set of lower and upper limits for both marginal posteriors and 2D
#'   contour plots that plot the values of parameter i along the x-axis.
#' @param y_lims_mat A matrix with two columns. The \emph{j}th row contains a
#'   new set of lower and upper limits for 2D contour plots that plot the values
#'   of parameter j along the y-axis.
#' @param param_num The number of parameters.
#' @param pic_width The dimensionless width of the new ggplot matrix object
#'   when saving.
#' @param aspect_ratio The aspect ratio of the new ggplot matrix object when
#'   saving.
#' @param pic_units The units to save the new ggplot matrix object in. E.g. cm.
#'   The default is cm.
#' @param save_plot A Boolean on whether the ggplot matrix is saved or not.
#'   Default is TRUE.
#' @param save_title A string containing the name that the ggplot matrix is
#'   saved under.
#' @param image_type A string containing the file type to save new ggplot
#'   matrix as. E.g. jpg or pdf.
#'
#' @return A square ggplot matrix object.
#'
#' @export
change_limits_ggmat <- function(plot_matrix,
                                cols_changed_vec,
                                x_lims_mat,
                                y_lims_mat,
                                param_num,
                                pic_width = 17.4,
                                aspect_ratio = 1.36,
                                pic_units = "cm",
                                save_plot = TRUE,
                                save_title = NA,
                                image_type = NA) {
  for (h in cols_changed_vec) {
    j <- h + 1
    while (j <= param_num) {
      plot_matrix[j, h] <- plot_matrix[j, h] +
        ggplot2::coord_cartesian(
          xlim = x_lims_mat[h, ],
          ylim = y_lims_mat[j, ]
        )
      j <- j + 1
    }

    plot_matrix[h, h] <- plot_matrix[h, h] +
      ggplot2::coord_cartesian(xlim = x_lims_mat[h, ])
  }

  if (save_plot == TRUE) {
    pic_height <- pic_width / aspect_ratio

    file_save_name <- paste0(save_title, ".", image_type)
    ggplot2::ggsave(file_save_name,
      plot = plot_matrix,
      dpi = 300,
      height = pic_height,
      width = pic_width,
      units = pic_units,
      cairo_pdf
    )
  }
  return(plot_matrix)
}