#' Calculate the prior ratio between two parameter vectors
#'
#' @description
#' This function calculates the prior probability of both the proposed 
#' parameter vector and the current position of the Markov
#' chains. The ratio of the probabilities is then taken.
#' @param prop_param_vec Proposed parameter values.
#' @param curr_param_vec Current position of the Markov chains.
#' @param mean_vec Means of the normal prior distributions.
#' @param sd_vec Standard deviations of the normal priors.
#'
#' @return A number, the prior probability ratio.
#' @export
#'
#' @examples
#' compute_prior_ratio(c(0.1, 6), c(0.2, 5.5), c(0.5, 7), c(0.2, 0.4))
compute_prior_ratio <- function(prop_param_vec,
                                curr_param_vec,
                                mean_vec,
                                sd_vec) {
  x <- prop_param_vec
  y <- curr_param_vec
  z <- mean_vec
  w <- sd_vec

  prelim_vec <- rep(0, times = length(x))

  for (i in 1:length(x)) {
    numerator <- stats::dnorm(x[i], mean = z[i], sd = w[i])
    denominator <- stats::dnorm(y[i], mean = z[i], sd = w[i])

    prelim_vec[i] <- numerator / denominator
  }

  prod(prelim_vec)
}

#' Calculate posterior-predictive ratio between two sets of predicted stresses
#'
#' @description
#' This function calculates the posterior-predictive ratio of the stresses found
#' by inputting both the proposed parameter vector and the current position
#' position of the Markov chains into the model.
#' @param expt_data The experimental stresses we fit to.
#' @param prop_sim_data The proposed predicted stresses.
#' @param curr_sim_data The current predicted stresses.
#' @param alpha_hyp The first hyperparameter of the hyperprior on 
#'   the noise variance.
#' @param beta_hyp The second hyperparameter of the hyperprior on 
#'   the noise variance.
#' @param prop_true_params The proposed values of the untransformed parameters.
#' @param curr_true_params The current values of the untransformed parameters.
#'
#' @return
#' A list containing the posterior-predictive ratio, as well as the value of the
#' posterior-predictive for the proposed parameter vector and the current 
#' position of the Markov chains.
#'
#' @export
compute_post_pred_ratio <- function(expt_data,
                                    prop_sim_data,
                                    curr_sim_data,
                                    alpha_hyp,
                                    beta_hyp,
                                    prop_true_params,
                                    curr_true_params) {
  sigma_mat <- diag(beta_hyp / alpha_hyp, nrow = length(expt_data))

  post_pred_num <- Rfast::dmvt(expt_data,
    mu     = prop_sim_data,
    sigma  = sigma_mat,
    nu     = 2 * alpha_hyp,
    logged = TRUE
  )
  post_pred_denom <- Rfast::dmvt(expt_data,
    mu     = curr_sim_data,
    sigma  = sigma_mat,
    nu     = 2 * alpha_hyp,
    logged = TRUE
  )

  # Compute the transformation factor to account for the transformation of
  # parameter values.
  prelim_vec <- log(prop_true_params) - log(curr_true_params)
  

  result_term <- exp(post_pred_num - post_pred_denom)

  result <- list(
    "post_pred_ratio" = result_term,
    "post_pred_num"   = post_pred_num,
    "post_pred_denom" = post_pred_denom
  )
  result
}

#' Calculate the transition probability ratio
#'
#' @param prop_param_vec Proposed parameter values.
#' @param curr_param_vec Current position of the Markov chains.
#' @param mean_vec Means of the normal prior distributions.
#' @param sd_vec Standard deviations of the normal priors.
#' @param expt_data The experimental stresses we fit to.
#' @param prop_sim_data The proposed predicted stresses.
#' @param curr_sim_data The current predicted stresses.
#' @param alpha_hyp The first hyperparameter of the hyperprior on the 
#'   noise variance.
#' @param beta_hyp The second hyperparameter of the hyperprior on the 
#'   noise variance.
#' @param prop_true_params The proposed values of the untransformed parameters.
#' @param curr_true_params The current values of the untransformed parameters.
#'
#' @return
#' The transition probability.
#'
#' @export
compute_trans_prob <- function(prop_param_vec,
                               curr_param_vec,
                               mean_vec,
                               sd_vec,
                               expt_data,
                               prop_sim_data,
                               curr_sim_data,
                               alpha_hyp,
                               beta_hyp,
                               prop_true_params,
                               curr_true_params) {
  prior_ratio <- compute_prior_ratio(
    prop_param_vec,
    curr_param_vec,
    mean_vec,
    sd_vec
  )

  if (prior_ratio == 0) {
    return(0.0)
  } else {
    prelim_list <- compute_post_pred_ratio(
      expt_data,
      prop_sim_data,
      curr_sim_data,
      alpha_hyp,
      beta_hyp,
      prop_true_params,
      curr_true_params
    )
    full_post_pred_ratio <- prelim_list[[1]]

    prelim_num <- full_post_pred_ratio * prior_ratio

    trans_prob <- min(c(1.0, prelim_num))

    return(trans_prob)
  }
}
