#' Run RWM MCMC for a given set of experimental stress-strain data
#'
#' @param strain_data The experimental stretch values.
#' @param stress_data The experimental engineering stress values.
#' @param starting_values Initial guesses for the parameters.
#' @param lower_cdf_values Parameter values at which the lower cumulative
#'   probability is reached.
#' @param upper_cdf_values Parameter values at which the higher cumulative
#'   probability is reached.
#' @param lower_cd Lower cumulative distribution.
#' @param upper_cd Higher cumulative distribution.
#' @param df_log_col_names Names for the columns of the transformed parameter
#'   data frame.
#' @param df_col_names Names for the columns of the untransformed parameter
#'   data frame.
#' @param par_num Number of model parameters.
#' @param block_length Number of iterations per block.
#' @param burn_in_length Number of iterations in the burn-in phase.
#' @param sim_num Number of simulations.
#' @param scale_cov_factor Scaling factor for the proposal covariance matrix.
#' @param accept_range Lower and upper satisfactory acceptance rates.
#' @param alpha_hyp First parameter of the inverse-gamma hyperprior.
#' @param beta_hyp Second parameter of the inverse-gamma hyperprior.
#' @param cov_calc_num Number of previous iterations to use to calculate the
#'   parameter covariances.
#' @param cov_calc_some Boolean determining whether only the last 
#'   \code{cov_calc_num} iterations are used to calculate parameter covariances 
#'   or whether all previous iterations are used.
#' @param change_cov_mat Boolean determining whether the proposal covariance
#'   matrix is changed or not.
#' @param print_iter An integer that determines after how many iterations we
#'   print the iteration number to the console.  
#' @param input_init_cov_mat Boolean on whether a user-designated initial
#'   covariance matrix is used or not.
#' @param init_cov_mat An initial covariance matrix.
#' @param fit_vec_num The number of closest fits that are retained in the
#'   matrix of closest fits.
#'
#' @return
#' A list containing the following terms:
#' 
#'  \itemize{
#'     \item a matrix with the transformed parameters found in the algorithm.
#'     \item a matrix with the untransformed parameter found in the algorithm
#'        and the average relative error obtained at the end of each iteration
#'        of the algorithm.
#'     \item a vector of the acceptance ratios in each block.
#'     \item the covariance matrix of the proposal distribution.
#'     \item the matrix of closest fits achieved in the algorithm.
#' }
#' @export
run_rwm_mcmc <- function(strain_data,
                         stress_data,
                         starting_values,
                         lower_cdf_values,
                         upper_cdf_values,
                         lower_cd,
                         upper_cd,
                         df_log_col_names,
                         df_col_names,
                         par_num = 4,
                         block_length = 100,
                         burn_in_length = 100000,
                         sim_num = 1000000,
                         scale_cov_factor = 0.95,
                         accept_range = c(0.184, 0.284),
                         alpha_hyp = 3,
                         beta_hyp = 0.3,
                         cov_calc_num = 100000,
                         cov_calc_some = FALSE,
                         change_cov_mat = TRUE,
                         print_iter = 10000,
                         input_init_cov_mat = FALSE,
                         init_cov_mat = diag(rep(1, par_num)) / 5,
                         fit_vec_num = 5) {
  # Calculate the means and standard deviations for our prior distributions in
  # the algorithm.
  means_and_sds_list <- compute_prior_params(
    smaller_values = lower_cdf_values,
    larger_values = upper_cdf_values,
    lower_cd = lower_cd,
    higher_cd = upper_cd
  )

  mean_vec <- means_and_sds_list[[1]]
  sd_vec <- means_and_sds_list[[2]]

  fits_mat <- matrix(10000000, nrow = fit_vec_num, ncol = (par_num + 2))

  accept_ratio_vec <- rep(1, (sim_num - 1) / block_length)

  log_params_mat <- matrix(0, ncol = par_num, nrow = sim_num)
  true_params_mat <- matrix(0, ncol = (par_num + 3), nrow = sim_num)

  first_param_vec <- transform_true_params(starting_values[1:par_num])

  log_params_mat[1, ] <- first_param_vec

  true_params_mat[1, 1:6] <- c(
    starting_values[1:par_num],
    starting_values[par_num - 1] + 1,
    sum(starting_values[(par_num - 1):par_num]) + 1
  )

  sim_stress <- calculate_st_tot_1pkstress(
    log_params_mat[1, 1],
    log_params_mat[1, 2],
    log_params_mat[1, 3],
    log_params_mat[1, 4],
    strain_data
  )
  ave_rel_error_init <- compute_ave_rel_err(stress_data, sim_stress)

  true_params_mat[1, 7] <- ave_rel_error_init

  fits_mat[1, ] <- c(1, ave_rel_error_init, true_params_mat[1, 1:par_num])

  total_blocks <- burn_in_length / block_length

  if (input_init_cov_mat == TRUE) {
    sigma_prop_dist <- init_cov_mat
  } else {
    sigma_prop_dist <- compute_proposal_cov_mat(1, diag(sd_vec))
  }

  cov_coeff <- 1

  prev_stress <- sim_stress

  for (i in 2:sim_num)
  {
    prop_vec <- Rfast::rmvnorm(1,
      mu    = log_params_mat[i - 1, 1:par_num],
      sigma = sigma_prop_dist
    )

    sim_stress <- calculate_st_tot_1pkstress(
      prop_vec[1],
      prop_vec[2],
      prop_vec[3],
      prop_vec[4],
      strain_data
    )

    prop_true_params <- transform_log_params(prop_vec)

    trans_prob <- compute_trans_prob(
      prop_vec,
      log_params_mat[i - 1, 1:par_num],
      mean_vec,
      sd_vec,
      stress_data,
      sim_stress,
      prev_stress,
      alpha_hyp,
      beta_hyp,
      prop_true_params[1:4],
      true_params_mat[(i - 1), 1:4]
    )

    if (stats::runif(1) <= trans_prob) {
      ave_rel_error <- compute_ave_rel_err(stress_data, sim_stress)
      
      log_params_mat[i, 1:par_num] <- prop_vec
      true_params_mat[i, 1:(par_num + 2)] <- prop_true_params
      true_params_mat[i, 7] <- ave_rel_error
      prev_stress <- sim_stress

      fits_mat <- alter_closest_fits_mat(
        fits_mat,
        fit_vec_num,
        ave_rel_error,
        i,
        true_params_mat[i, 1:par_num]
      )
    } else {
      log_params_mat[i, ] <- log_params_mat[i - 1, ]
      true_params_mat[i, ] <- true_params_mat[i - 1, ]
    }
    if ((i - 1) %% block_length == 0) {
      # We calculate the covariance matrix for the proposal distribution at the
      # end of each block. We use the acceptance ratio of the block to decide
      # how to calculate the proposal distribution.
      accept_ratio_vec[(i - 1) / block_length] <-
        (length(unique(log_params_mat[(i - block_length + 1):i, 2])) - 1) /
          block_length

      cond <- change_cov_mat && (i - 1) <= burn_in_length
      if (cond == TRUE) {
        if (accept_ratio_vec[(i - 1) / block_length] < min(accept_range)) {
          cov_coeff <- cov_coeff * (scale_cov_factor)^2
        } else if (accept_ratio_vec[(i - 1) / block_length] > 
                   max(accept_range)) {
          cov_coeff <- cov_coeff * (2 - scale_cov_factor)^2
        }

        if (cov_calc_some == TRUE && cov_calc_num < (i - 1)) {
          j <- i - cov_calc_num
          prelim_cov_mat <- stats::cov(log_params_mat[(j + 1):i, 1:par_num]) +
            0.00001 * diag(par_num)
        } else {
          prelim_cov_mat <- stats::cov(log_params_mat[2:i, 1:par_num]) +
            0.00001 * diag(par_num)
        }

        sigma_prop_dist <- compute_proposal_cov_mat(
          cov_coeff,
          prelim_cov_mat
        )
      }
    }
    if ((i - 1) %% print_iter == 0) {
      print(i)
    }
  }

  log_params_df <- as.data.frame(log_params_mat[(burn_in_length + 2):sim_num, ])
  colnames(log_params_df) <- df_log_col_names

  true_par_df <- as.data.frame(true_params_mat[(burn_in_length + 2):sim_num, ])
  colnames(true_par_df) <- df_col_names

  fits_df <- as.data.frame(fits_mat)

  return_object <- list(
    "log_dataframe" = log_params_df,
    "true_dataframe" = true_par_df,
    "acceptRatio" = accept_ratio_vec,
    "propCovMatrix" = sigma_prop_dist,
    "best_fit_vecs" = fits_df
  )
  return_object
}
