#' Calculates the Cauchy stress of the non-collagenous matrix (NCM)
#'
#' @param nu The natural logarithm of (1-phi)*mu.
#' @param lambda The stretch acting on the tissue.
#'
#' @return
#' A number, corresponding to the Cauchy stress acting on the NCM.
#' @export
#'
#' @examples
#' calculate_ncm_cstress(1, 1.2)
calculate_ncm_cstress <- function(nu, lambda) {
  exp(nu) * (lambda^2 - 1 / (lambda))
}


#' First function that calculates the Cauchy stress of the collagen fibrils
#'
#' @description
#' Function that calculates the contribution to the Cauchy stress from the
#' collagen fibrils when the stretch is greater than a and less than or equal
#' to the mode, which lies half-way between a and b.
#'
#' @param eta The natural logarithm of phi*E.
#' @param tau The natural logarithm of a-1.
#' @param lambda The stretch acting on the tissue.
#'
#' @return
#' A number, corresponding to the Cauchy stress acting on the collagen
#' fibrils.
#' @export
#'
#' @examples
#' calculate_st_fib_cstress_1(6, -3, 1.2)
calculate_st_fib_cstress_1 <- function(eta,
                                       tau,
                                       lambda) {
  term1 <- lambda^2 / 2
  term2 <- -(1 + exp(tau)) * lambda * log(lambda / (1 + exp(tau)))
  term3 <- -(1 + exp(tau))^2 / 2

  exp(eta)*(term1 + term2 + term3)
}


#' Second function that calculates the Cauchy stress of the collagen fibrils
#'
#' @description
#' Function that calculates the contribution to the Cauchy stress from the
#' collagen fibrils when the stretch is greater than the mode, which lies
#' half-way between a and b, but less than or equal to b.
#'
#' @param eta The natural logarithm of phi*E.
#' @param tau The natural logarithm of a-1.
#' @param rho The natural logarithm of b-a.
#' @param lambda The stretch acting on the tissue.
#'
#' @return
#' A number, corresponding to the Cauchy stress acting on the collagen
#' fibrils.
#' @export
#'
#' @examples
#' calculate_st_fib_cstress_2(6, -3, -2, 1.2)
calculate_st_fib_cstress_2 <- function(eta,
                                       tau,
                                       rho,
                                       lambda) {
  term1 <- (1 + exp(tau)) * lambda * log(1 + exp(tau))
  term2 <- -(1 + exp(tau))^2 / 2
  term3 <- -lambda^2 / 2
  term4 <- (1 + exp(tau) + exp(rho)) * lambda * log(lambda)
  term5 <- -(2 + 2 * exp(tau) + exp(rho)) * lambda *
    log(1 + exp(tau) + exp(rho) / 2)
  term6 <- (2 + 2 * exp(tau) + exp(rho))^2 / 2
  term7 <- -(1 + exp(tau) + exp(rho) / 2)^2

  exp(eta)*(term1 + term2 + term3 + term4 + term5 + term6 + term7)
}


#' Third function that calculates the Cauchy stress of the collagen fibrils
#'
#' @description
#' Function that calculates the contribution to the Cauchy stress from the
#' collagen fibrils when the stretch is greater than b.
#'
#' @param eta The natural logarithm of phi*E.
#' @param tau The natural logarithm of a-1.
#' @param rho The natural logarithm of b-a.
#' @param lambda The stretch acting on the tissue.
#'
#' @return
#' A number, corresponding to the Cauchy stress acting on the collagen
#' fibrils.
#' @export
#'
#' @examples
#' calculate_st_fib_cstress_3(6, -3, -2, 1.2)
calculate_st_fib_cstress_3 <- function(eta,
                                       tau,
                                       rho,
                                       lambda) {
  term1 <- -(2 + 2 * exp(tau) + exp(rho)) * lambda *
    log(1 + exp(tau) + exp(rho) / 2)
  term2 <- (2 + 2 * exp(tau) + exp(rho))^2 / 2
  term3 <- (1 + exp(tau)) * lambda * log(1 + exp(tau))
  term4 <- (1 + exp(tau) + exp(rho)) * lambda * log(1 + exp(tau) + exp(rho))
  term5 <- -(1 + exp(tau) + exp(rho) / 2)^2
  term6 <- -(1 + exp(tau))^2 / 2
  term7 <- -(1 + exp(tau) + exp(rho))^2 / 2

  exp(eta)*(term1 + term2 + term3 + term4 + term5 + term6 + term7)
}


#' Calculates the Cauchy stress of the collagen fibrils for a set of stretches
#'
#' @description
#' Function that determines which expression must be evaluated to calculate
#' the stress acting on the fibrils at a given stretch, and which evaluates
#' each function for a vector of stretch values.
#'
#' @param eta The natural logarithm of phi*E.
#' @param tau The natural logarithm of a-1.
#' @param rho The natural logarithm of b-a.
#' @param lambda The vector of stretches acting on the tissue.
#'
#' @return
#' A vector, corresponding to the Cauchy stress acting on the collagen
#' fibrils at each stretch.
#' @export
#'
#' @examples
#' calculate_st_fib_cstress(6, -3, -2, c(1.2, 1.25, 1.3))
calculate_st_fib_cstress <- function(eta,
                                     tau,
                                     rho,
                                     lambda) {
  a <- 1 + exp(tau)
  n <- a + exp(rho) / 2
  b <- n + exp(rho) / 2
  l <- lambda

  prelim <- ifelse(a > l,
    0,
    ifelse(a <= l & l <= n,
      calculate_st_fib_cstress_1(eta, tau, l),
      ifelse(n < l & l <= b,
        calculate_st_fib_cstress_2(eta, tau, rho, l),
        calculate_st_fib_cstress_3(eta, tau, rho, l)
      )
    )
  )
  prelim * (4 * exp(-2 * rho))
}

#' Calculates the Cauchy stress of a tissue for a vector of stretches
#'
#' @param nu The natural logarithm of (1-phi) mu.
#' @param eta The natural logarithm of phi*E.
#' @param tau The natural logarithm of a-1.
#' @param rho The natural logarithm of b-a.
#' @param lambda The vector of stretches acting on the tissue.
#'
#' @return
#' A vector, corresponding to the Cauchy stress acting on the tissue at each
#' stretch measured.
#' @export
#'
#' @examples
#' calculate_st_tot_cstress(1, 6, -1, -0.1, c(1.2, 1.25, 1.3))
calculate_st_tot_cstress <- function(nu,
                                     eta, 
                                     tau,
                                     rho,
                                     lambda) {
  prelim <- calculate_ncm_cstress(nu, lambda) +
    calculate_st_fib_cstress(eta, tau, rho, lambda)

  prelim
}

#' Calculates the engineering stress of a tissue for a vector of stretches
#'
#' @param nu The natural logarithm of (1-phi)mu.
#' @param eta The natural logarithm of phi*E.
#' @param tau The natural logarithm of a-1.
#' @param rho The natural logarithm of b-a.
#' @param lambda The stretch acting on the tissue.
#'
#' @return
#' A vector, corresponding to the engineering stress acting on the collagen
#' fibrils at each stretch.
#' @export
#'
#' @examples
#' calculate_st_tot_1pkstress(1, 6, -3, -2, c(1.2, 1.25, 1.3))
calculate_st_tot_1pkstress <- function(nu,
                                       eta,
                                       tau,
                                       rho,
                                       lambda) {
  calculate_st_tot_cstress(nu, eta, tau, rho, lambda) / lambda
}
