#include <iostream>
#include <vector>
#include <cmath>
#include "RCppArmadillo.h"
#include "Faddeeva.h"
using namespace Rcpp;

// [[Rcpp::depends(RcppArmadillo]]

//' Add an accepted parameter vector to the matrix detailing the closest fits
//' 
//' Each time a proposed parameter vector is accepted in the Random-Walk 
//' Metropolis Markov chain Monte Carlo algorithm, we ascertain the average
//' relative error to the experimental data set. If the error is less than one
//' of the vectors in the matrix, then the newly accepted parameter vector
//' replaces the vector in the matrix that produces the worst fit to data.
//'
//' @param current_mat A matrix of the accepted parameter vectors that
//'   achieve the best fits to experimental data in the algorithm so far.
//' @param n_row The number of parameter vectors that we save in the matrix.
//' @param ave_error The average relative error between the experimental data
//'   and the predicted stresses found with the accepted parameter vector.
//' @param iter An integer, the iteration in the algorithm that the parameter
//'   vector was accepted.
//' @param param_vec The accepted parameter vector.
//'
//' @return A matrix of the transformed parameter vectors that have been
//'   accepted and produced the best n_row fits to data so far in the algorithm.
// [[Rcpp::export]]
arma::mat rcpp_alter_closest_fits_mat(arma::mat current_mat,
                                      arma::uword n_row,
                                      double ave_error,
                                      double iter,
                                      arma::vec param_vec) {
  
  arma::vec average_errors(n_row, arma::fill::zeros);
  average_errors = current_mat.col(1);
  
  arma::mat result_mat(n_row, current_mat.n_cols, arma::fill::zeros);
  
  if(ave_error < arma::max(average_errors)){
    arma::rowvec row_vec(current_mat.n_cols, arma::fill::zeros);
    
    row_vec(0) = iter;
    
    row_vec(1) = ave_error;
    row_vec(arma::span(2, current_mat.n_cols - 1)) = param_vec.t();
    current_mat.row(n_row-1) = row_vec;
    
    arma::vec sort_vec = current_mat.col(1);
    
    arma::uvec indices = sort_index(sort_vec);
    
    for(arma::uword i = 0; i < sort_vec.n_elem; i++){
      result_mat.row(i) = current_mat.row(indices[i]);
    }
  } else{
    result_mat = current_mat;
  }
  return result_mat;
}
