#include <iostream>
#include <vector>
#include <cmath>
#include "RCppArmadillo.h"
#include "Isostress.h"
using namespace Rcpp;

// [[Rcpp::depends(RcppArmadillo)]]


//' First function for the stress on the collagen fibrils (general tri. dist.)
//'
//' @description
//' Function that calculates the contribution to the Cauchy stress from the
//' collagen fibrils when the stretch is greater than a and less than or equal
//' to the mode, c.
//'
//' @param eta The natural logarithm of phi*E, where E is the Young's modulus
//'   of the collagen fibrils and phi is the collagen fibril volume fraction.
//' @param tau The natural logarithm of a-1.
//' @param xi The natural logarithm of c-a.
//' @param chi The natural logarithm of b-c, where b is the maximum 
//'   fibril length.
//' @param lambda The stretch acting on the tissue.
//'
//' @return
//' A number, corresponding to the Cauchy stress acting on the collagen
//' fibrils.
//' @export
//'
//' @examples
//' rcpp_calc_gt_fib_cstress1(6, -3, -3, -3, 1.07)
// [[Rcpp::export]]
double rcpp_calc_gt_fib_cstress1(double eta,
                                 double tau,
                                 double xi,
                                 double chi,
                                 double lambda){
  double result = 0.0;
  
  double term1 = lambda * lambda;
  double term2 = -2.0 * (1.0 + exp(tau)) * lambda * 
    log(lambda / (1.0 + exp(tau)));
  double term3 = -((1.0 + exp(tau)) * (1.0 + exp(tau)));
  
  result = (term1 + term2 + term3) / ((exp(xi) + exp(chi)) * exp(xi));
  
  return exp(eta) * result;
}


//' Second function for the stress on the collagen fibrils (general tri. dist.)
//'
//' @description
//' Function that calculates the contribution to the Cauchy stress from the
//' collagen fibrils when the stretch is greater than the mode, c, but less 
//' than or equal to b, the maximum fibril length.
//'
//' @param eta The natural logarithm of phi*E, where E is the Young's modulus
//'   of the collagen fibrils and phi is the collagen fibril volume fraction.
//' @param tau The natural logarithm of a-1, where a is the minimum collagen
//'   fibril length.
//' @param xi The natural logarithm of c-a.
//' @param chi The natural logarithm of b-c.
//' @param lambda The stretch acting on the tissue.
//'
//' @return
//' A number, corresponding to the Cauchy stress acting on the collagen
//' fibrils.
//' @export
//'
//' @examples
//' rcpp_calc_gt_fib_cstress2(6, -3, -3, -3, 1.12)
// [[Rcpp::export]]
double rcpp_calc_gt_fib_cstress2(double eta,
                                 double tau,
                                 double xi,
                                 double chi,
                                 double lambda){
  double term1 = (1.0 + exp(tau) + exp(xi)) * (1.0 + exp(tau) + exp(xi));
  double term2 = -((1.0 + exp(tau)) * (1.0 + exp(tau)));
  double term3 = 2.0 * (1.0 + exp(tau)) * lambda * log(1.0 + exp(tau));
  double term4 = -(2.0 * (1.0 + exp(tau) + exp(xi)) * lambda * 
                   log(1.0 + exp(tau) + exp(xi)));
  double term5 = -lambda * lambda;
  double term6 = 2.0 * (1.0 + exp(tau) + exp(xi) + exp(chi)) * lambda * 
    log(lambda);
  
  double denom1 = exp(xi) * exp(chi);
  double denom2 = (exp(xi) + exp(chi)) * exp(xi);
  double denom3 = (exp(xi) + exp(chi)) * exp(chi);
  
  double prelim1 = term1 / denom1;
  double prelim2 = term2 / denom2;
  double prelim3 = term3 / denom2;
  double prelim4 = term4 / denom1;
  double prelim5 = term5 / denom3;
  double prelim6 = term6 / denom3;
  
  double prelim = prelim1 + prelim2 + prelim3 + prelim4 + prelim5 + prelim6;
  
  return exp(eta)*prelim;
}


//' Third function for the stress on the collagen fibrils (general tri. dist.)
//'
//' @description
//' Function that calculates the contribution to the Cauchy stress from the
//' collagen fibrils when the stretch is greater than b, the maximum collagen
//' fibril length.
//'
//' @param eta The natural logarithm of phi*E, where E is the Young's modulus
//'   of the collagen fibrils and phi is the collagen fibril volume fraction.
//' @param tau The natural logarithm of a-1, where a is the minimum length of 
//'   the collagen fibrils.
//' @param xi The natural logarithm of c-a, where c is the modal length of the
//'   collagen fibrils.
//' @param chi The natural logarithm of b-c.
//' @param lambda The stretch acting on the tissue.
//'
//' @return
//' A number, corresponding to the Cauchy stress acting on the collagen
//' fibrils.
//' @export
//'
//' @examples
//' rcpp_calc_gt_fib_cstress3(6, -3, -3, -3, 1.2)
// [[Rcpp::export]]
double rcpp_calc_gt_fib_cstress3(double eta,
                                 double tau,
                                 double xi,
                                 double chi,
                                 double lambda){
  double term1 = -1.0;
  double term2 = (2.0 * (1.0 + exp(tau)) * lambda * log(1.0 + exp(tau))) / 
    ((exp(xi) + exp(chi)) * exp(xi));
  double term3 = (2.0 * (1.0 + exp(tau) + exp(xi) + exp(chi)) * lambda * 
                  log(1.0 + exp(tau) + exp(xi) + exp(chi))) / 
                  ((exp(xi) + exp(chi)) * exp(chi));
  double term4 = -(2.0 * (1.0 + exp(tau) + exp(xi)) * lambda * 
                   log(1.0 + exp(tau) + exp(xi))) / 
                   (exp(xi) * exp(chi));
  
  double prelim = term1 + term2 + term3 + term4;
  return exp(eta)*prelim;
}


//' Computes collagen fibril stress for a set of stretches (general tri. dist.)
//'
//' @description
//' Function that first determines which expression must be evaluated to 
//' calculate the stress acting on the fibrils at a given stretch. The function
//' also evaluates the stress, for a given parameter vector, at a vector of 
//' stretch values.  
//'
//' @param eta The natural logarithm of phi*E, where E is the Young's modulus
//'   of the collagen fibrils and phi is the collagen fibril volume fraction.
//' @param tau The natural logarithm of a-1, where a is the minimum collagen
//'   fibril length.
//' @param xi The natural logarithm of c-a, where c is the modal collagen fibril
//'   length.
//' @param chi The natural logarithm of b-c, where b is the maximum collagen
//'   fibril length.
//' @param lambda The vector of stretches that act on the tissue.
//'
//' @return
//' A vector, corresponding to the Cauchy stress acting on the collagen
//' fibrils at each stretch.
//' @export
//'
//' @examples
//' rcpp_calc_gt_fib_cstress(6, -3, -3, -3, c(1.06, 1.1, 1.2, 1.3))
// [[Rcpp::export]]
arma::vec rcpp_calc_gt_fib_cstress(double eta,
                                   double tau,
                                   double xi,
                                   double chi,
                                   arma::vec lambda){
  double a = 1.0 + exp(tau);
  double n = a + exp(xi);
  double b = n + exp(chi);
  
  int n_elem = lambda.size();
  
  arma::vec result(n_elem, arma::fill::zeros);
  
  for(int i = 0; i < n_elem; i++){
    if(a <= lambda[i] && lambda[i] <= n){
      result[i] = rcpp_calc_gt_fib_cstress1(eta, tau, xi, chi, lambda[i]);
    } else if(n < lambda[i] && lambda[i] <= b){
      result[i] = rcpp_calc_gt_fib_cstress2(eta, tau, xi, chi, lambda[i]);
    } else if(b < lambda[i]){
      result[i] = rcpp_calc_gt_fib_cstress3(eta, tau, xi, chi, lambda[i]);
    }
  }
  
  return result;
}


//' Computes tissue Cauchy stress for a set of stretches (general tri. dist.)
//'
//' @param nu The natural logarithm of (1-phi)*mu, where mu is the shear modulus
//'   of the non-collagenous matrix (NCM) and (1-phi) is the volume fraction
//'   of the NCM in the tissue.
//' @param eta The natural logarithm of phi*E, where E is the Young's modulus
//'   of the collagen fibrils and phi is the collagen fibril volume fraction.
//' @param tau The natural logarithm of a-1, where a is the minimum collagen
//'   fibril length.
//' @param xi The natural logarithm of c-a, where c is the modal collagen fibril
//'   length.
//' @param chi The natural logarithm of b-c, where b is the maximum collagen
//'   fibril length.
//' @param lambda The vector of stretches acting on the tissue.
//'
//' @return
//' A vector, corresponding to the Cauchy stress acting on the tissue at each
//' stretch measured.
//' @export
//'
//' @examples
//' rcpp_calc_gt_tot_cstress(1, 6, -3, -3, -3, c(1.08, 1.10, 1.2, 1.25, 1.3))
// [[Rcpp::export]]
arma::vec rcpp_calc_gt_tot_cstress(double nu,
                                   double eta,
                                   double tau,
                                   double xi,
                                   double chi,
                                   arma::vec lambda){
  int n_elem = lambda.size();
  
  arma::vec result(n_elem, arma::fill::zeros);
  
  result = rcpp_calc_ncm_cstress(nu, lambda) + 
    rcpp_calc_gt_fib_cstress(eta, tau, xi, chi, lambda);
  
  return result;
}


//' Computes tissue engineering stress for a set of stretches (gen. tri. dist.)
//'
//' @param nu The natural logarithm of (1-phi)*mu, where mu is the shear modulus
//'   of the non-collagenous matrix (NCM) and (1-phi) is the volume fraction
//'   of the NCM in the tissue.
//' @param eta The natural logarithm of phi*E, where E is the Young's modulus
//'   of the collagen fibrils and phi is the collagen fibril volume fraction.
//' @param tau The natural logarithm of a-1, where a is the minimum collagen
//'   fibril length.
//' @param xi The natural logarithm of c-a, where c is the modal collagen fibril
//'   length.
//' @param chi The natural logarithm of b-c, where b is the maximum collagen
//'   fibril length.
//' @param lambda The vector of stretches acting on the tissue.
//'
//' @return
//' A vector, corresponding to the engineering stress acting on the collagen
//' fibrils at each stretch.
//' @export
//'
//' @examples
//' rcpp_calc_gt_tot_1pkstress(1, 6, -3, -3, -3, c(1.08, 1.1, 1.2, 1.25, 1.3))
// [[Rcpp::export]]
arma::vec rcpp_calc_gt_tot_1pkstress(double nu,
                                     double eta,
                                     double tau,
                                     double xi,
                                     double chi,
                                     arma::vec lambda){
  return rcpp_calc_gt_tot_cstress(nu, eta, tau, xi, chi, lambda) / lambda;
}
