#include <iostream>
#include <vector>
#include <cmath>
#include "RCppArmadillo.h"
using namespace Rcpp;

// [[Rcpp::depends(RcppArmadillo)]]

//' Calculate logarithms of the model parameters
//'
//' @param true_params A vector of, in order, (1-phi)*mu, phi*E, a-1, b-a.
//'
//' @return A vector with the logarithms of the model parameters.
//' @export
//'
//' @examples
//' rcpp_transf_true_params(c(5, 600, 0.02, 0.08))
// [[Rcpp::export]]
arma::vec rcpp_transf_true_params(arma::vec true_params) {
  return arma::log(true_params);
}


//' Calculate the model parameters from their logarithms
//'
//' @description
//' This function takes the logarithms of the four model parameters and creates
//' the following six-element vector:
//' \itemize{
//' \item elements 1-4 contain the model parameters.
//' \item element 5 is the minimal recruitment stretch, a.
//' \item element 6 is the maximal recruitment stretch, b.
//' }
//'
//' @param log_params A vector of, in order, nu, eta, tau, rho.
//'
//' @return A vector containing six elements that correspond to the four 
//'   fit parameters used in the RWM algorithm and the second distribution 
//'   parameters. (See the description above for more details.)
//' @export
//'
//' @examples
//' rcpp_transf_log_params(c(1.61, 6.40, -3.91, -2.53))
// [[Rcpp::export]]
arma::vec rcpp_transf_log_params(arma::vec log_params) {
  int n  = log_params.n_elem;
  int n2 = n + 2;
  arma::vec result(n2, arma::fill::zeros);
  
  for(int i = 0; i < n; i++){
    result(i) = exp(log_params(i));
  }
  
  result(n) = result(n-2) + 1.0;
  result(n+1) = result(n-1) + result(n);
  
  return result;
}
