#include <iostream>
#include <vector>
#include <cmath>
#include "RCppArmadillo.h"
using namespace Rcpp;

// [[Rcpp::depends(RcppArmadillo)]]

//' Calculate prior ratio between two parameter vectors
//'
//' @description
//' This function calculates the prior probability of both the proposed
//' parameter vector and the current position of the Markov
//' chains. The ratio of the probabilities is then taken.
//' @param prop_params Proposed parameter values.
//' @param curr_params Current position of the Markov chains.
//' @param mean_vec Means of the normal prior distributions.
//' @param sd_vec Standard deviations of the normal priors.
//'
//' @return A number, the prior probability ratio.
//'
//' @examples
//' rcpp_compute_prior_ratio(c(0.1, 6), c(0.2, 5.5), c(0.5, 7), c(0.2, 0.4))
// [[Rcpp::export]]
double rcpp_compute_prior_ratio(arma::vec prop_params, 
                                arma::vec curr_params,
                                arma::vec mean_vec,
                                arma::vec sd_vec){
  int n_elem = prop_params.size();
  
  int return_log = 0;
  
  arma::vec result(n_elem, arma::fill::ones);
  
  for(int i = 0; i < n_elem; i++){
    double numerator = R::dnorm(prop_params(i),
                                mean_vec(i),
                                sd_vec(i),
                                return_log);
    
    double denominator = R::dnorm(curr_params(i),
                                  mean_vec(i),
                                  sd_vec(i),
                                  return_log);
    
    result(i) = numerator / denominator;
  }

  return prod(result);
}


//' A multivariate Student's t distribution coded in C++
//'
//' @description
//' This function calculates the log-probability of obtaining the experimental
//' stresses according to a Student's t distribution with mean values given
//' by the predicted stresses of the model.
//' @param obs_vec Experimental stresses.
//' @param mu_vec Predicted stresses.
//' @param scale_mat A scale matrix for this multivariate distribution.
//' @param nu The number of degrees of freedom.
//'
//' @return A number, the probability for the vector obs_vec.
// [[Rcpp::export]]
double rcpp_compute_dmvt(arma::rowvec obs_vec, 
                         arma::rowvec mu_vec, 
                         arma::mat scale_mat, 
                         double nu){
  
  int p = obs_vec.n_elem;
  
  double term1, term2, term3, term4 = 0.0;
  double term5, term6b, term6, prelim_result = 0.0;
  
  term1 = lgamma((nu + p)/2);
  term2 = lgamma(nu / 2);
  term3 = 0.5 * p * log(nu);
  term4 = 0.5 * p* log(arma::datum::pi);
  term5 = 0.5 * log(det(scale_mat));
  
  arma::rowvec term6a(p, arma::fill::zeros);
  
  term6a = ((obs_vec - mu_vec) * arma::inv(scale_mat));
  term6b = sum(term6a % (obs_vec - mu_vec));
  term6  = 0.5 * (nu + p) * log(1.0 + term6b / nu);
  
  prelim_result = term1 - term2 - term3 - term4 - term5 - term6;
  
  return prelim_result;
}


//' Calculate posterior-predictive ratio between two sets of predicted stresses
//'
//' @description
//' This function calculates the posterior-predictive of the stresses found by
//' inputting the proposed parameter vector into the model and the current 
//' position of the Markov chains into the model.
//' @param expt_data The experimental stresses we fit to.
//' @param prop_sim_data The proposed predicted stresses.
//' @param curr_sim_data The current predicted stresses.
//' @param alpha The first hyperparameter of the hyperprior on the 
//'   noise variance.
//' @param beta The second hyperparameter of the hyperprior on the 
//'   noise variance.
//' @param prop_true_params The proposed values of the untransformed parameters.
//' @param curr_true_params The current values of the untransformed parameters.
//'
//' @return
//' A vector containing the posterior-predictive ratio, as well as the value of 
//' the posterior-predictive for the proposed parameter vector and the current 
//' position of the Markov chains.
// [[Rcpp::export]]
arma::vec rcpp_calc_post_pred_rat(arma::rowvec expt_data,
                                  arma::rowvec prop_sim_data,
                                  arma::rowvec curr_sim_data,
                                  double alpha,
                                  double beta,
                                  arma::rowvec prop_true_params,
                                  arma::rowvec curr_true_params){
  int n = expt_data.n_elem;
  
  arma::mat sigma_mat(n, n, arma::fill::eye);
  
  sigma_mat = sigma_mat * (beta / alpha);
  
  double num, denom = 0.0;
  
  // We calculate the log of the probability, according to the Student's t
  // distribution for now. We transform it to the actual probability at the end 
  // of the function.
  num = rcpp_compute_dmvt(expt_data,
                          prop_sim_data,
                          sigma_mat,
                          2*alpha);
  
  denom = rcpp_compute_dmvt(expt_data,
                            curr_sim_data,
                            sigma_mat,
                            2*alpha);
  
  // Compute the transformation factor to account for the transformation of
  // parameter values.
  int n_params = prop_true_params.n_elem;
  arma::rowvec prelim_vec(n_params, arma::fill::ones);
  prelim_vec = arma::log(prop_true_params) - arma::log(curr_true_params);
  
  
  
  arma::vec result(3, arma::fill::zeros);
  
  // 
  result(0) = exp(num - denom);
  result(1) = num;
  result(2) = denom;
  
  return result;
}


//' Calculate the transition probability ratio
//'
//' @param prop_params Proposed parameter values.
//' @param curr_params Current position of the Markov chains.
//' @param mean_vec Means of the normal prior distributions.
//' @param sd_vec Standard deviations of the normal priors.
//' @param expt_data The experimental stresses we fit to.
//' @param prop_sim_data The proposed predicted stresses.
//' @param curr_sim_data The current predicted stresses.
//' @param alpha The first hyperparameter of the hyperprior on the 
//'   noise variance.
//' @param beta The second hyperparameter of the hyperprior on the 
//'   noise variance.
//' @param prop_true_params The proposed values of the untransformed parameters.
//' @param curr_true_params The current values of the untransformed parameters.
//'
//' @return
//' A vector containing the transition and prior probability ratios and the
//' likelihood ratio.
// [[Rcpp::export]]
arma::vec rcpp_compute_trans_prob(arma::vec prop_params, 
                                  arma::vec curr_params,
                                  arma::vec mean_vec,
                                  arma::vec sd_vec,
                                  arma::rowvec expt_data,
                                  arma::rowvec prop_sim_data,
                                  arma::rowvec curr_sim_data,
                                  double alpha,
                                  double beta,
                                  arma::rowvec prop_true_params,
                                  arma::rowvec curr_true_params){
  double p_r = rcpp_compute_prior_ratio(prop_params,
                                        curr_params,
                                        mean_vec,
                                        sd_vec);
  
  double t_p, prelim_t_p = 0.0;
  double l_r = 0.0;
  
  arma::vec prelim(4);
  arma::vec t_p_vec(2);
  
  if(p_r == 0.0){
    t_p = 0.0;
  } else{
    prelim = rcpp_calc_post_pred_rat(expt_data,
                                     prop_sim_data,
                                     curr_sim_data,
                                     alpha,
                                     beta,
                                     prop_true_params,
                                     curr_true_params);
    
    l_r = prelim(0);
    
    prelim_t_p = l_r*p_r;
    
    t_p_vec(0) = 1.0;
    t_p_vec(1) = prelim_t_p;
    
    t_p = min(t_p_vec);
  }
  
  arma::vec result(5, arma::fill::ones);
  
  result(0) = t_p;
  result(1) = p_r;
  result(2) = l_r;
  result(3) = prelim(1);
  result(4) = prelim(2);
  
  return result;
}