#include <iostream>
#include <vector>
#include <cmath>
#include "Isostress.h"
#include "Symtrics1.h"
#include "Symtrics2.h"
#include "Symtrics3.h"
#include "Symtrifibstr.h"
#include "Symtritotstr.h"
#include "Symtri1pkstr.h"
#include "Tflogparams.h"
#include "Tftrueparams.h"
#include "Priorratio.h"
#include "Dmvt.h"
#include "Ppredratio.h"
#include "Transprob.h"
#include "Closestfit.h"
#include "Calcaverelerr.h"
#include "Strunmcmc.h"
#include "RCppArmadillo.h"
// [[Rcpp::depends(RcppArmadillo)]]

using namespace Rcpp;

//' Run RWM MCMC for the symmetric triangular distribution
//'
//' @param stretch_expt The experimental stretch ratios.
//' @param stress_expt The experimentally observed engineering stress values,
//'   in megapascals.
//' @param starting_values The starting values of the parameters (1-phi)*mu, 
//'   phi*\emph{E}, \emph{a}-1, and \emph{b}-\emph{a}.
//' @param mean_vec A vector of means for the normal prior distributions of the
//'   transformed parameters.
//' @param sd_vec A vector of the standard deviations for the normal prior
//'   distributions of the transformed parameters.
//' @param param_num The number of fit parameters.
//' @param block_length An integer that represents the number of simulations
//'   that make up one block in the algorithm.
//' @param burn_in_length An integer that represents the number of simulations
//'   performed in the burn-in phase.
//' @param sim_num An integer that signifies the total number of simulations
//'   performed in the algorithm.
//' @param scale_cov_factor A double corresponding to the scale factor, beta,
//'   used in the adaptation of the proposal distribution covariance matrix.
//' @param accept_range A vector containing the minimum and maximum acceptance
//'   rates that are deemed to be acceptable for the efficient running of the 
//'   algorithm.
//' @param alpha_hyp A double corresponding to the parameter alpha of the 
//'   hyperprior.
//' @param beta_hyp A double corresponding to the parameter beta of the 
//'   hyperprior.
//' @param cov_calc_num An integer that defines how many simulations we take 
//'   the results from to adapt the covariance matrix of the proposal 
//'   distribution in the burn-in phase. We always take the most recent 
//'   \emph{N} simulations.
//' @param cov_calc_some A Boolean that determines whether we use the results
//'   of all previous simulations or not when adapting the covariance matrix of 
//'   the proposal distribution in the burn-in phase.
//' @param change_cov_mat A Boolean that determines whether we change the 
//'   covariance matrix of the proposal distribution or not.
//' @param print_iter An integer that determines after how many simulations do
//'   we print the simulation number to the console (to show the progress of 
//'   the algorithm).
//' @param fit_vec_num An integer that determines how many parameter vectors we
//'   collect in a matrix that collates the parameter vectors that achieve the
//'   best fit to data. The fit to data is determined in terms of the average
//'   relative error between the predicted stress and the experimental stress.
//'
//' @return
//' A list containing the following terms:
//' 
//'  \itemize{
//'    \item a matrix with the transformed parameters found in the algorithm.
//'    \item a matrix with the untransformed parameter found in the algorithm
//'      and the average relative error obtained at the end of each iteration
//'      of the algorithm.
//'    \item a vector of the acceptance ratios in each block.
//'    \item the covariance matrix of the proposal distribution.
//'    \item the matrix of closest fits achieved in the algorithm.
//' }
// [[Rcpp::export]]
Rcpp::List rcpp_st_run_mcmc(arma::vec stretch_expt,
                            arma::vec stress_expt,
                            arma::vec starting_values,
                            arma::vec mean_vec,
                            arma::vec sd_vec,
                            arma::uword param_num,
                            arma::uword block_length,
                            unsigned long int burn_in_length,
                            unsigned long int sim_num,
                            double scale_cov_factor,
                            arma::vec accept_range,
                            double alpha_hyp,
                            double beta_hyp,
                            unsigned long int cov_calc_num,
                            bool cov_calc_some,
                            bool change_cov_mat,
                            unsigned long int print_iter,
                            arma::uword fit_vec_num) {
  arma::uword n_stretch = stretch_expt.n_elem;
  
  arma::mat fits_mat(fit_vec_num, param_num + 2, arma::fill::ones);
  fits_mat = fits_mat * 1000000;
  
  arma::uword n_blocks = (sim_num - 1) / block_length;
  
  arma::vec accept_ratio(n_blocks, arma::fill::zeros);
  
  arma::mat logpara_mat(sim_num, param_num, arma::fill::zeros);
  arma::mat truepara_mat(sim_num, param_num + 3, arma::fill::zeros);
  arma::vec algo_params(param_num, arma::fill::zeros);
  
  algo_params = rcpp_transf_true_params(starting_values);
  logpara_mat.row(0) = algo_params.t();
  
  truepara_mat(0, arma::span(0, param_num - 1)) = starting_values.t();
  truepara_mat(0, param_num) = starting_values[param_num - 2] + 1.0;
  truepara_mat(0, param_num + 1) = 
    sum(starting_values.subvec(param_num - 2, param_num - 1)) + 1.0;
  
  arma::vec sim_stress(n_stretch, arma::fill::ones);
  
  sim_stress = rcpp_calc_st_tot_1pkstress(algo_params[0], 
                                          algo_params[1],
                                          algo_params[2],
                                          algo_params[3],
                                          stretch_expt);
  
  double ave_rel_error = rcpp_calc_ave_rel_err(stress_expt, sim_stress);
  
  truepara_mat(0, param_num + 2) = ave_rel_error;
  
  fits_mat(0, 0) = 1.0;
  fits_mat(0, 1) = ave_rel_error;
  fits_mat(0, arma::span(2, param_num + 1)) = 
    truepara_mat(0, arma::span(0, param_num - 1));
  
  arma::mat cov_mat(param_num, param_num, arma::fill::zeros);
  arma::mat sigma_prop_dist(param_num, param_num, arma::fill::zeros);
  
  sigma_prop_dist = 0.2 * arma::diagmat(sd_vec);
  
  double cov_coeff = 1.0;
  
  arma::vec prev_stress(n_stretch, arma::fill::ones);
  prev_stress = sim_stress;
  
  arma::vec prop_vector(param_num, arma::fill::zeros);
  arma::vec mean_prop_vec(param_num, arma::fill::zeros);
  arma::vec t_p_vec(5, arma::fill::zeros);
  double move_prob = 0.0;
  
  double rand_num = 0.0;
  
  unsigned long int acceptances = 0;
  
  unsigned long int accept_index = 0;
  
  arma::rowvec prop_true_params(param_num + 2, arma::fill::zeros);
  arma::rowvec prop_tparams(param_num, arma::fill::zeros);
  
  for(unsigned long int i = 1; i < sim_num; i++){
    mean_prop_vec = logpara_mat.row(i - 1).t();
    
    prop_vector = arma::mvnrnd(mean_prop_vec, sigma_prop_dist);
    
    prop_true_params = rcpp_transf_log_params(prop_vector).t();
    
    prop_tparams = prop_true_params.subvec(0, param_num - 1);
    
    sim_stress = rcpp_calc_st_tot_1pkstress(prop_vector[0],
                                            prop_vector[1],
                                            prop_vector[2],
                                            prop_vector[3],
                                            stretch_expt);
    
    t_p_vec = rcpp_compute_trans_prob(prop_vector,
                                      mean_prop_vec,
                                      mean_vec,
                                      sd_vec,
                                      stress_expt.t(),
                                      sim_stress.t(),
                                      prev_stress.t(),
                                      alpha_hyp,
                                      beta_hyp,
                                      prop_tparams,
                                      truepara_mat((i - 1), 
                                                   arma::span(0, 
                                                              param_num - 1)));
    move_prob = t_p_vec[0];
    rand_num = arma::randu<double>();
    
    if(rand_num <= move_prob){
      ave_rel_error = rcpp_calc_ave_rel_err(stress_expt, sim_stress);
      
      logpara_mat.row(i) = prop_vector.t();
      truepara_mat(i, arma::span(0, param_num + 1)) = prop_true_params;
      truepara_mat(i, param_num + 2) = ave_rel_error;
      prev_stress = sim_stress;
      acceptances = acceptances + 1;
      fits_mat = rcpp_alter_closest_fits_mat(fits_mat,
                                             fit_vec_num,
                                             ave_rel_error,
                                             i,
                                             prop_tparams.t());
    } else{
      logpara_mat.row(i) = logpara_mat.row(i - 1);
      truepara_mat.row(i) = truepara_mat.row(i - 1);
    }
    
    if(i % block_length == 0){
       // We calculate the covariance matrix for the proposal distribution at
       // the end of each block. We use the acceptance ratio of the block to
       // decide how to calculate the proposal distribution.
      accept_index = i / block_length - 1;
      
      // Redefine the integers acceptances and block_length as doubles in order
      // to be able to divide the former by the latter and obtain a double
      accept_ratio[accept_index] = (double)acceptances / (double)block_length;
      
      acceptances = 0;
      
      bool prelim_cond = (i <= burn_in_length);
      
      bool cond = (change_cov_mat && prelim_cond);
      
      if(cond == true){
        if(accept_ratio[accept_index] < min(accept_range)){
          cov_coeff = cov_coeff * scale_cov_factor * scale_cov_factor;
        } else if(accept_ratio[accept_index] > max(accept_range)){
          cov_coeff = cov_coeff * (2.0 - scale_cov_factor) * 
            (2.0 - scale_cov_factor);
        }
        if(cov_calc_some == true && cov_calc_num < i){
          unsigned long int j = i - cov_calc_num;
          cov_mat = arma::cov(logpara_mat(arma::span(j, i - 1), 
                                          arma::span(0, param_num - 1))) + 
                                            0.00001 * arma::eye(param_num, 
                                                                param_num);
        } else{
          cov_mat = arma::cov(logpara_mat(arma::span(1, i - 1), 
                                          arma::span(0, param_num - 1))) + 
                                            0.00001 * arma::eye(param_num, 
                                                                param_num);
        }
        sigma_prop_dist = cov_coeff * cov_mat;  
      }
    }
    if (i % print_iter == 0){
      Rcout << "iteration: " << i << "\n";
    }
  }
  List result_list = List::create(logpara_mat, truepara_mat, accept_ratio,
                                  sigma_prop_dist, fits_mat);
  return result_list;
}
