#include <iostream>
#include <vector>
#include <cmath>
#include "RCppArmadillo.h"
#include "Symtrics3.h"
using namespace Rcpp;

// [[Rcpp::depends(RcppArmadillo)]]

//' Calculates the Cauchy stress of the non-collagenous matrix (NCM)
//'
//' @param nu The natural logarithm of (1-phi)*mu.
//' @param lambda A vector of tissue stretches.
//'
//' @return
//' A vector, corresponding to the Cauchy stress acting on the NCM at each
//' stretch.
//' @export
//'
//' @examples
//' rcpp_calc_ncm_cstress(1, c(1.05, 1.1, 1.15, 1.2))
// [[Rcpp::export]]
arma::vec rcpp_calc_ncm_cstress(double nu,
                                arma::vec lambda){
  int n_elem = lambda.size();
  
  arma::vec result(n_elem, arma::fill::zeros);
  
  return exp(nu) * (lambda % lambda - 1 / lambda);
}


//' First function that calculates the Cauchy stress of the collagen fibrils
//'
//' @description
//' Function that calculates the contribution to the Cauchy stress from the
//' collagen fibrils when the stretch is greater than a and less than or equal
//' to the mode, which lies half-way between a and b.
//'
//' @param eta The natural logarithm of phi*E.
//' @param tau The natural logarithm of a-1.
//' @param lambda The stretch acting on the tissue.
//'
//' @return
//' A number, corresponding to the Cauchy stress acting on the collagen
//' fibrils.
//' @export
//'
//' @examples
//' rcpp_calc_st_fib_cstress1(6, -3, 1.2)
// [[Rcpp::export]]
double rcpp_calc_st_fib_cstress1(double eta,
                                 double tau,
                                 double lambda){
  double result = 0.0;
  
  double term1  = (lambda * lambda) / 2;
  double term2a = -(1 + exp(tau)) * lambda;
  double term2b = log(lambda / (1 + exp(tau))); 
  double term3  = -((1 + exp(tau)) * (1 + exp(tau))) / 2;
  
  result = exp(eta) * (term1 + term2a * term2b + term3);
  
  return result;
}


//' Second function that calculates the Cauchy stress of the collagen fibrils
//'
//' @description
//' Function that calculates the contribution to the Cauchy stress from the
//' collagen fibrils when the stretch is greater than the mode, which lies
//' half-way between a and b, but less than or equal to b.
//'
//' @param eta The natural logarithm of phi*E.
//' @param tau The natural logarithm of a-1.
//' @param rho The natural logarithm of b-a.
//' @param lambda The stretch acting on the tissue.
//'
//' @return
//' A number, corresponding to the Cauchy stress acting on the collagen
//' fibrils.
//' @export
//'
//' @examples
//' rcpp_calc_st_fib_cstress2(6, -3, -2, 1.2)
// [[Rcpp::export]]
double rcpp_calc_st_fib_cstress2(double eta,
                                 double tau,
                                 double rho,
                                 double lambda){
  double term1 = (1 + exp(tau)) * lambda * log(1 + exp(tau));
  double term2 = -((1 + exp(tau)) * (1 + exp(tau))) / 2;
  double term3 = -(lambda * lambda) / 2;
  double term4 = (1 + exp(tau) + exp(rho)) * lambda * log(lambda);
  double term5 = -(2 + 2 * exp(tau) + exp(rho)) * lambda * 
    log(1 + exp(tau) + exp(rho) / 2);
  double term6 = ((2 + 2 * exp(tau) + exp(rho)) * (2 + 2 * 
                  exp(tau) + exp(rho))) / 4;
  
  double prelim = term1 + term2 + term3 + term4 + term5 + term6;
  
  return exp(eta)*prelim;
}


//' Third function that calculates the Cauchy stress of the collagen fibrils
//'
//' @description
//' Function that calculates the contribution to the Cauchy stress from the
//' collagen fibrils when the stretch is greater than b.
//'
//' @param eta The natural logarithm of phi*E.
//' @param tau The natural logarithm of a-1.
//' @param rho The natural logarithm of b-a.
//' @param lambda The stretch acting on the tissue.
//'
//' @return
//' A number, corresponding to the Cauchy stress acting on the collagen
//' fibrils.
//' @export
//'
//' @examples
//' rcpp_calc_st_fib_cstress3(6, -3, -2, 1.2)
// [[Rcpp::export]]
double rcpp_calc_st_fib_cstress3(double eta,
                                 double tau, 
                                 double rho,
                                 double lambda){
  double term1 = -(2 + 2 * exp(tau) + exp(rho)) * lambda * 
    log(1 + exp(tau) + exp(rho) / 2);
  double term2 = (1 + exp(tau)) * lambda * log(1 + exp(tau));
  double term3 = (1 + exp(tau) + exp(rho)) * lambda * 
    log(1 + exp(tau) + exp(rho));
  double term4 = -exp(2 * rho) / 4;
  
  double prelim = term1 + term2 + term3 + term4;
  return exp(eta)*prelim;
}


//' Calculates the Cauchy stress of the collagen fibrils for a set of stretches
//'
//' @description
//' Function that determines which expression must be evaluated to calculate
//' the stress acting on the fibrils at a given stretch, and which evaluates
//' each function for a vector of stretch values.
//'
//' @param eta The natural logarithm of phi*E.
//' @param tau The natural logarithm of a-1.
//' @param rho The natural logarithm of b-a.
//' @param lambda The vector of stretches that act on the tissue.
//'
//' @return
//' A vector, corresponding to the Cauchy stress acting on the collagen
//' fibrils at each stretch.
//' @export
//'
//' @examples
//' rcpp_calc_st_fib_cstress(6, -3, -2, c(1.1, 1.2, 1.3))
// [[Rcpp::export]]
arma::vec rcpp_calc_st_fib_cstress(double eta,
                                   double tau,
                                   double rho,
                                   arma::vec lambda){
  double a = 1.0 + exp(tau);
  double n = a + exp(rho)/2;
  double b = n + exp(rho)/2;
  
  int n_elem = lambda.size();
  
  arma::vec result(n_elem, arma::fill::zeros);
  
  for(int i = 0; i < n_elem; i++){
    if(a <= lambda[i] && lambda[i] <= n){
      result[i] = rcpp_calc_st_fib_cstress1(eta, tau, lambda[i]);
    } else if(n < lambda[i] && lambda[i] <= b){
      result[i] = rcpp_calc_st_fib_cstress2(eta, tau, rho, lambda[i]);
    } else if(b < lambda[i]){
      result[i] = rcpp_calc_st_fib_cstress3(eta, tau, rho, lambda[i]);
    }
  }
  
  return result * 4 * exp(-2 * rho);
}


//' Calculates the Cauchy stress of a tissue for a vector of stretches
//'
//' @param nu The natural logarithm of (1-phi)*mu.
//' @param eta The natural logarithm of phi*E.
//' @param tau The natural logarithm of a-1.
//' @param rho The natural logarithm of b-a.
//' @param lambda The vector of stretches acting on the tissue.
//'
//' @return
//' A vector, corresponding to the Cauchy stress acting on the tissue at each
//' stretch measured.
//' @export
//'
//' @examples
//' rcpp_calc_st_tot_cstress(1, 6, -3, -2, c(1.2, 1.25, 1.3))
// [[Rcpp::export]]
arma::vec rcpp_calc_st_tot_cstress(double nu,
                                   double eta,
                                   double tau,
                                   double rho,
                                   arma::vec lambda){
  int n_elem = lambda.size();
  
  arma::vec result(n_elem, arma::fill::zeros);
  
  result = rcpp_calc_ncm_cstress(nu, lambda) + 
    rcpp_calc_st_fib_cstress(eta, tau, rho, lambda);
  
  return result;
}


//' Calculates the engineering stress of a tissue for a vector of stretches
//'
//' @param nu The natural logarithm of (1-phi)*mu.
//' @param eta The natural logarithm of phi*E.
//' @param tau The natural logarithm of a-1.
//' @param rho The natural logarithm of b-a.
//' @param lambda The vector of stretches acting on the tissue.
//'
//' @return
//' A vector, corresponding to the Cauchy stress acting on the collagen
//' fibrils at each stretch.
//' @export
//'
//' @examples
//' rcpp_calc_st_tot_1pkstress(1, 6, -3, -2, c(1.2, 1.25, 1.3))
// [[Rcpp::export]]
arma::vec rcpp_calc_st_tot_1pkstress(double nu,
                                     double eta,
                                     double tau,
                                     double rho,
                                     arma::vec lambda){
  return rcpp_calc_st_tot_cstress(nu, eta, tau, rho, lambda) / lambda;
}
